@@ -3,7 +3,7 @@
  *
  *   Interactive command loop, C and Tcl callable.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXcmdloop.c,v 8.12 1999/03/31 06:37:43 markd Exp $
+ * $Id: tclXcmdloop.c,v 8.0 1996/11/21 00:24:01 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -68,11 +68,6 @@ SyncSignalErrorHandler _ANSI_ARGS_((Tcl_Interp *interp,
 static void
 AsyncCommandHandlerDelete _ANSI_ARGS_((ClientData clientData));
 
-static int 
-TclX_CommandloopObjCmd _ANSI_ARGS_((ClientData clientData, 
-                                    Tcl_Interp *interp,
-                                    int objc,
-                                    Tcl_Obj *CONST objv[]));
 
 /*-----------------------------------------------------------------------------
  * IsSetVarCmd --
@@ -89,16 +84,25 @@ static int
 IsSetVarCmd (command)
     char  *command;
 {
-    Tcl_Parse tclParse;
-    int numWords;
+    CONST char  *nextPtr;
+    int    wordCnt;
 
     if ((!STRNEQU (command, "set", 3)) || (!ISSPACE (command [3])))
         return FALSE;  /* Quick check */
 
-    Tcl_ParseCommand(NULL, command, -1, 1, &tclParse);
-    numWords = tclParse.numWords;
-    Tcl_FreeParse(&tclParse);
-    return numWords > 2 ? TRUE : FALSE;
+    /*
+     * Loop to count the words in the command.
+     */
+    wordCnt = 0;
+    nextPtr = command;
+    while (*nextPtr != '\0') {
+        nextPtr = TclWordEnd (nextPtr, FALSE, NULL);
+        nextPtr++;  /* Character after the word */
+        while ((*nextPtr != '\0') && (ISSPACE (*nextPtr)))
+            nextPtr++;
+        wordCnt++;
+    }
+    return wordCnt > 2 ? TRUE : FALSE;
 }
 
 /*-----------------------------------------------------------------------------
@@ -108,7 +112,7 @@ IsSetVarCmd (command)
  *
  * Parameters:
  *   o interp (I) - A pointer to the interpreter.  Result of command should be
- *     in interp result.
+ *     in interp->result.
  *   o intResult (I) - The integer result returned by Tcl_Eval.
  *   o checkCmd (I) - If not NULL and the command was sucessful, check to
  *     set if this is a "set" command setting a variable.  If so, don't echo
@@ -122,7 +126,6 @@ TclX_PrintResult (interp, intResult, checkCmd)
     char       *checkCmd;
 {
     Tcl_Channel stdoutChan,  stderrChan;
-    char *resultStr;
 
     /*
      * If the command was supplied and it was a successful set of a variable,
@@ -137,11 +140,10 @@ TclX_PrintResult (interp, intResult, checkCmd)
     if (intResult == TCL_OK) {
         if (stdoutChan == NULL)
             return;
-        resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (interp), NULL);
-        if (resultStr [0] != '\0') {
+        if (interp->result [0] != '\0') {
             if (stderrChan != NULL)
                 Tcl_Flush (stderrChan);
-            TclX_WriteStr (stdoutChan, resultStr);
+            TclX_WriteStr (stdoutChan, interp->result);
             TclX_WriteNL (stdoutChan);
             Tcl_Flush (stdoutChan);
         }
@@ -158,9 +160,8 @@ TclX_PrintResult (interp, intResult, checkCmd)
         } else {
             sprintf (msg, "Bad return code (%d): ", intResult);
         }
-        resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (interp), NULL);
         TclX_WriteStr (stderrChan, msg);
-        TclX_WriteStr (stderrChan, resultStr);
+        TclX_WriteStr (stderrChan, interp->result);
         TclX_WriteNL (stderrChan);
         Tcl_Flush (stderrChan);
     }
@@ -191,7 +192,7 @@ OutputPrompt (interp, topLevel, prompt1, prompt2)
     char       *prompt1;
     char       *prompt2;
 {
-    char *promptHook, *resultStr;
+    char *promptHook;
     int result, useResult, promptDone = FALSE;
     Tcl_Channel stdoutChan, stderrChan;
 
@@ -232,16 +233,15 @@ OutputPrompt (interp, topLevel, prompt1, prompt2)
 
     if (promptHook != NULL) {
         result = Tcl_Eval (interp, promptHook);
-        resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (interp), NULL);
         if (result == TCL_ERROR) {
             if (stderrChan != NULL) {
                 TclX_WriteStr (stderrChan, "Error in prompt hook: ");
-                TclX_WriteStr (stderrChan, resultStr);
+                TclX_WriteStr (stderrChan, interp->result);
                 TclX_WriteNL (stderrChan);
             }
         } else {
             if (useResult && (stdoutChan != NULL))
-                TclX_WriteStr (stdoutChan, resultStr);
+                TclX_WriteStr (stdoutChan, interp->result);
             promptDone = TRUE;
         }
     } 
@@ -317,7 +317,7 @@ AsyncCommandHandler (clientData, mask)
 {
     asyncLoopData_t *dataPtr = (asyncLoopData_t *) clientData;
     int code;
-    char *cmd, *resultStr;
+    char *cmd;
 
     /*
      * Make sure that we are the current signal error handler.  This
@@ -369,10 +369,7 @@ AsyncCommandHandler (clientData, mask)
     code = Tcl_RecordAndEval (dataPtr->interp, cmd, TCL_EVAL_GLOBAL);
     Tcl_CreateChannelHandler (dataPtr->channel, TCL_READABLE,
                               AsyncCommandHandler, clientData);
-
-    resultStr = Tcl_GetStringFromObj (Tcl_GetObjResult (dataPtr->interp),
-                                      NULL);
-    if (resultStr [0] != '\0') {
+    if (dataPtr->interp->result [0] != '\0') {
         if (dataPtr->options & TCLX_CMDL_INTERACTIVE) {
             TclX_PrintResult (dataPtr->interp, code, cmd);
         }
@@ -634,8 +631,8 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
                 gotInterrupted = TRUE; 
                 continue;  /* Process signals above */
             }
-            TclX_AppendObjResult (interp, "command input error on stdin: ",
-                                  Tcl_PosixError (interp), (char *) NULL);
+            Tcl_AppendResult (interp, "command input error on stdin: ",
+                              Tcl_PosixError (interp), (char *) NULL);
             return TCL_ERROR;
         }
 
@@ -674,7 +671,7 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
 }
 
 /*-----------------------------------------------------------------------------
- * Tcl_CommandloopObjCmd --
+ * Tcl_CommandloopCmd --
  *    Implements the commandloop command:
  *       commandloop -async -interactive on|off|tty -prompt1 cmd
  *                   -prompt2 cmd -endcommand cmd
@@ -682,53 +679,49 @@ TclX_CommandLoop (interp, options, endCommand, prompt1, prompt2)
  *   Standard TCL results.
  *-----------------------------------------------------------------------------
  */
-static int
-TclX_CommandloopObjCmd (clientData, interp, objc, objv)
+int
+Tcl_CommandloopCmd (clientData, interp, argc, argv)
     ClientData  clientData;
     Tcl_Interp *interp;
-    int         objc;
-    Tcl_Obj    *CONST objv[];
+    int         argc;
+    char      **argv;
 {
     int options = 0, async = FALSE, argIdx, interactive;
-    char *argStr,  *endCommand = NULL;
+    char *endCommand = NULL;
     char *prompt1 = NULL, *prompt2 = NULL;
 
     interactive = isatty (0);
-    for (argIdx = 1; argIdx < objc; argIdx++) {
-        argStr = Tcl_GetStringFromObj (objv [argIdx], NULL);
-        if (argStr [0] != '-')
-            break;
-        if (STREQU (argStr, "-async")) {
+    for (argIdx = 1; (argIdx < argc) && (argv [argIdx][0] == '-'); argIdx++) {
+        if (STREQU (argv [argIdx], "-async")) {
             async = TRUE;
-        } else if (STREQU (argStr, "-prompt1")) {
-            if (argIdx == objc - 1)
+        } else if (STREQU (argv [argIdx], "-prompt1")) {
+            if (argIdx == argc - 1)
                 goto argRequired;
-            prompt1 = Tcl_GetStringFromObj (objv [++argIdx], NULL);;
-        } else if (STREQU (argStr, "-prompt2")) {
-            if (argIdx == objc - 1)
+            prompt1 = argv [++argIdx];
+        } else if (STREQU (argv [argIdx], "-prompt2")) {
+            if (argIdx == argc - 1)
                 goto argRequired;
-            prompt2 = Tcl_GetStringFromObj (objv [++argIdx], NULL);
-        } else if (STREQU (argStr, "-interactive")) {
-            if (argIdx == objc - 1)
+            prompt2 = argv [++argIdx];
+        } else if (STREQU (argv [argIdx], "-interactive")) {
+            if (argIdx == argc - 1)
                 goto argRequired;
             argIdx++;
-            argStr = Tcl_GetStringFromObj (objv [argIdx], NULL);
-            if (STREQU (argStr, "tty")) {
+            if (STREQU (argv [argIdx], "tty")) {
                 interactive = TRUE;
             } else {
-                if (Tcl_GetBooleanFromObj (interp, objv [argIdx],
-                                           &interactive) != TCL_OK)
+                if (Tcl_GetBoolean (interp, argv [argIdx],
+                                    &interactive) != TCL_OK)
                     return TCL_ERROR;
             }
-        } else if (STREQU (argStr, "-endcommand")) {
-            if (argIdx == objc - 1)
+        } else if (STREQU (argv [argIdx], "-endcommand")) {
+            if (argIdx == argc - 1)
                 goto argRequired;
-            endCommand = Tcl_GetStringFromObj (objv [++argIdx], NULL);
+            endCommand = argv [++argIdx];
         } else {
             goto unknownOption;
         }
     }
-    if (argIdx != objc)
+    if (argIdx != argc)
         goto wrongArgs;
 
     if (interactive)
@@ -750,41 +743,25 @@ TclX_CommandloopObjCmd (clientData, interp, objc, objv)
 
 
     /*
-     * Argument error message generation.  argStr should contain the
+     * Argument error message generation.  argv [argIdx] should contain the
      * option being processed.
      */
   argRequired:
-    TclX_AppendObjResult (interp, "argument required for ", argStr,
-                          " option", (char *) NULL);
+    Tcl_AppendResult (interp, "argument required for ", argv [argIdx],
+                      " option", (char *) NULL);
     return TCL_ERROR;
 
   unknownOption:
-    TclX_AppendObjResult (interp, "unknown option \"", argStr,
-                          "\", expected one of \"-async\", ",
-                          "\"-interactive\", \"-prompt1\", \"-prompt2\", ",
-                          " or \"-endcommand\"", (char *) NULL);
+    Tcl_AppendResult (interp, "unknown option \"", argv [argIdx],
+                      "\", expected one of \"-async\", ",
+                      "\"-interactive\", \"-prompt1\", \"-prompt2\", ",
+                      " or \"-endcommand\"", (char *) NULL);
     return TCL_ERROR;
     
   wrongArgs:
-    TclX_WrongArgs (interp, objv [0],
-                    "?-async? ?-interactive on|off|tty? ?-prompt1 cmd? ?-prompt2 cmd? ?-endcommand cmd?");
+    Tcl_AppendResult (interp, tclXWrongArgs, argv [0], " ?-async? ",
+                      "?-interactive on|off|tty? ?-prompt1 cmd? ",
+                      "?-prompt2 cmd? ?-endcommand cmd?",
+                      (char *) NULL);
     return TCL_ERROR;
 }
-
-/*-----------------------------------------------------------------------------
- * TclX_CmdloopInit --
- *     Initialize the coommandloop command.
- *-----------------------------------------------------------------------------
- */
-void
-TclX_CmdloopInit (interp)
-    Tcl_Interp *interp;
-{
-    Tcl_CreateObjCommand (interp,
-                          "commandloop",
-                          TclX_CommandloopObjCmd, 
-                          (ClientData) NULL,
-                          (Tcl_CmdDeleteProc*) NULL);
-    
-}
-
