@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXdebug.c,v 7.0 1996/06/16 05:30:11 markd Exp $
+ * $Id: tclXdebug.c,v 8.0 1996/11/21 00:24:02 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -79,6 +79,12 @@ CmdTraceRoutine _ANSI_ARGS_((ClientData    clientData,
                              int           argc,
                              char        **argv));
 
+static int
+Tcl_CmdtraceCmd _ANSI_ARGS_((ClientData    clientData,
+                             Tcl_Interp   *interp,
+                             int           argc,
+                             char        **argv));
+
 static void
 DebugCleanUp _ANSI_ARGS_((ClientData  clientData,
                           Tcl_Interp *interp));
@@ -133,7 +139,7 @@ PrintStr (channel, string, numChars, quoted)
             Tcl_Write (channel, &(string [idx]), 1);
         }
     }
-    if (numChars < strlen (string))
+    if (numChars < (int) strlen (string))
         Tcl_Write (channel, "...", 3);
     if (quoted) 
         Tcl_Write (channel, "}", 1);
@@ -496,7 +502,7 @@ Tcl_CmdtraceCmd (clientData, interp, argc, argv)
     }
     infoPtr->traceHolder = Tcl_CreateTrace (interp,
                                             infoPtr->depth,
-                                            CmdTraceRoutine,
+                                            (Tcl_CmdTraceProc*)CmdTraceRoutine,
                                             (ClientData) infoPtr);
     return TCL_OK;
 
@@ -569,7 +575,7 @@ Tcl_InitDebug (interp)
     Tcl_CallWhenDeleted (interp, DebugCleanUp, (ClientData) infoPtr);
 
     Tcl_CreateCommand (interp, "cmdtrace", Tcl_CmdtraceCmd, 
-                       (ClientData) infoPtr, (void (*)()) NULL);
+                       (ClientData) infoPtr, (Tcl_CmdDeleteProc*) NULL);
 }
 
 
