@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXfilecmds.c,v 7.0 1996/06/16 05:30:18 markd Exp $
+ * $Id: tclXfilecmds.c,v 8.0 1996/11/21 00:24:04 markd Exp $
  *-----------------------------------------------------------------------------
  */
 /* 
@@ -45,8 +45,8 @@
 #include "tclExtdInt.h"
 
 static char *FILE_ID_OPT = "-fileid";
-static char *FILE_ID_NOT_AVAIL =
-    "the -fileid option is not available on this system";
+static char *TCL_TRANSLATION_OPT = "-translation";
+static char *TCL_EOFCHAR_OPT = "-eofchar";
 
 /*
  * Prototypes of internal functions.
@@ -69,9 +69,12 @@ TruncateByPath  _ANSI_ARGS_((Tcl_Interp  *interp,
                              off_t        newSize));
 
 static int
-TruncateByHandle  _ANSI_ARGS_((Tcl_Interp  *interp,
-                               char        *fileHandle,
-                               off_t        newSize));
+ReadDirCallback _ANSI_ARGS_((Tcl_Interp  *interp,
+                             char        *path,
+                             char        *fileName,
+                             int          caseSensitive,
+                             ClientData   clientData));
+
 
 /*-----------------------------------------------------------------------------
  * Tcl_PipeCmd --
@@ -89,11 +92,7 @@ Tcl_PipeCmd (clientData, interp, argc, argv)
     int         argc;
     char      **argv;
 {
-    int fileNums [2];
-    Tcl_File file;
-    Tcl_Channel chans [2];
-
-    chans [0] = chans [1] = NULL;
+    Tcl_Channel channels [2];
 
     if (!((argc == 1) || (argc == 3))) {
         Tcl_AppendResult (interp, tclXWrongArgs, argv[0], 
@@ -101,27 +100,22 @@ Tcl_PipeCmd (clientData, interp, argc, argv)
         return TCL_ERROR;
     }
 
-    if (TclX_OSpipe (interp, fileNums) != TCL_OK)
+    if (TclXOSpipe (interp, channels) != TCL_OK)
         return TCL_ERROR;
 
-    chans [0] = TclX_SetupFileEntry (interp,  fileNums [0], TCL_READABLE,
-                                     FALSE);
-    chans [1] = TclX_SetupFileEntry (interp,  fileNums [1], TCL_WRITABLE,
-                                     FALSE);
-
     if (argc == 1)
         Tcl_AppendResult (interp,
-                          Tcl_GetChannelName (chans [0]), " ",
-                          Tcl_GetChannelName (chans [1]),
+                          Tcl_GetChannelName (channels [0]), " ",
+                          Tcl_GetChannelName (channels [1]),
                           (char *) NULL);
     else {
         if (Tcl_SetVar (interp, argv[1], 
-                        Tcl_GetChannelName (chans [0]),
+                        Tcl_GetChannelName (channels [0]),
                         TCL_LEAVE_ERR_MSG) == NULL)
             goto errorExit;
 
         if (Tcl_SetVar (interp, argv[2], 
-                        Tcl_GetChannelName (chans [1]),
+                        Tcl_GetChannelName (channels [1]),
                         TCL_LEAVE_ERR_MSG) == NULL)
             goto errorExit;
     }
@@ -129,14 +123,8 @@ Tcl_PipeCmd (clientData, interp, argc, argv)
     return TCL_OK;
 
   errorExit:
-    if (chans [0] != NULL) 
-        Tcl_Close (NULL, chans [0]);
-    else
-        close (fileNums [0]);
-    if (chans [1] != NULL) 
-        Tcl_Close (NULL, chans [1]);
-    else
-        close (fileNums [1]);
+    Tcl_Close (NULL, channels [0]);
+    Tcl_Close (NULL, channels [1]);
     return TCL_ERROR;
 }
 
@@ -216,9 +204,10 @@ Tcl_CopyfileCmd (clientData, interp, argc, argv)
 #define TCLX_COPY_MAX_BYTES  2
 
     Tcl_Channel inChan, outChan;
-    int inTrans, outTrans;
     long totalBytesToRead, totalBytesRead;
     int argIdx, copyMode, translate, saveErrno;
+    Tcl_DString inChanTrans, outChanTrans;
+    Tcl_DString inChanEOF, outChanEOF;
 
     /*
      * Parse arguments.
@@ -240,7 +229,7 @@ Tcl_CopyfileCmd (clientData, interp, argc, argv)
             if (Tcl_GetLong (interp, argv [argIdx],
                              &totalBytesToRead) != TCL_OK)
                 return TCL_ERROR;
-        } else if (STREQU (argv [1], "-maxbytes")) {
+        } else if (STREQU (argv [argIdx], "-maxbytes")) {
             copyMode = TCLX_COPY_MAX_BYTES;
             argIdx++;
             if (argIdx >= argc) {
@@ -252,7 +241,7 @@ Tcl_CopyfileCmd (clientData, interp, argc, argv)
             if (Tcl_GetLong (interp, argv [argIdx],
                              &totalBytesToRead) != TCL_OK)
                 return TCL_ERROR;
-        } else if (STREQU (argv [1], "-translate")) {
+        } else if (STREQU (argv [argIdx], "-translate")) {
             translate = TRUE;
         } else {
             Tcl_AppendResult (interp, "invalid argument \"", argv [argIdx],
@@ -269,28 +258,41 @@ Tcl_CopyfileCmd (clientData, interp, argc, argv)
         return TCL_ERROR;
     }
 
+    Tcl_DStringInit (&inChanTrans);
+    Tcl_DStringInit (&outChanTrans);
+    Tcl_DStringInit (&inChanEOF);
+    Tcl_DStringInit (&outChanEOF);
 
     /*
      * Get the channels.  Unless translation is enabled, save the current
-     * translation mode and put the files in binary mode.
+     * translation mode and put the files in binary mode and clear the
+     * EOF character.
      */
     inChan = TclX_GetOpenChannel (interp, argv [argIdx], TCL_READABLE);
     if (inChan == NULL)
-        return TCL_ERROR;
+        goto errorExit;
     outChan = TclX_GetOpenChannel (interp, argv [argIdx + 1], TCL_WRITABLE);
     if (outChan == NULL)
-        return TCL_ERROR;
+        goto errorExit;
 
     if (!translate) {
-        inTrans = TclX_GetChannelOption (inChan, TCLX_COPT_TRANSLATION);
-        if (TclX_SetChannelOption (interp, inChan, TCLX_COPT_TRANSLATION,
-                                   TCLX_TRANSLATE_BINARY) != TCL_OK)
-            return TCL_ERROR;
-
-        outTrans = TclX_GetChannelOption (outChan, TCLX_COPT_TRANSLATION);
-        if (TclX_SetChannelOption (interp, outChan, TCLX_COPT_TRANSLATION,
-                                   TCLX_TRANSLATE_BINARY) != TCL_OK)
-            return TCL_ERROR;
+        Tcl_GetChannelOption (inChan, TCL_TRANSLATION_OPT, &inChanTrans);
+        Tcl_GetChannelOption (outChan, TCL_TRANSLATION_OPT, &outChanTrans);
+        Tcl_GetChannelOption (inChan, TCL_EOFCHAR_OPT, &inChanEOF);
+        Tcl_GetChannelOption (outChan, TCL_EOFCHAR_OPT, &outChanEOF);
+        
+        if (Tcl_SetChannelOption (interp, inChan, TCL_TRANSLATION_OPT,
+                                  "binary") != TCL_OK)
+            goto errorExit;
+        if (Tcl_SetChannelOption (interp, outChan, TCL_TRANSLATION_OPT,
+                                  "binary") != TCL_OK)
+            goto errorExit;
+        if (Tcl_SetChannelOption (interp, inChan, TCL_EOFCHAR_OPT,
+                                  "") != TCL_OK)
+            goto errorExit;
+        if (Tcl_SetChannelOption (interp, outChan, TCL_EOFCHAR_OPT,
+                                  "") != TCL_OK)
+            goto errorExit;
     }
 
     /*
@@ -304,20 +306,25 @@ Tcl_CopyfileCmd (clientData, interp, argc, argv)
         saveErrno = Tcl_GetErrno ();
 
     if (!translate) {
-        if (TclX_SetChannelOption (interp, inChan, TCLX_COPT_TRANSLATION,
-                                   inTrans) != TCL_OK)
-            return TCL_ERROR;
-
-        if (TclX_SetChannelOption (interp, outChan, TCLX_COPT_TRANSLATION,
-                                   outTrans) != TCL_OK)
-            return TCL_ERROR;
+        if (Tcl_SetChannelOption (interp, inChan, TCL_TRANSLATION_OPT,
+                                  inChanTrans.string) != TCL_OK)
+            goto errorExit;
+        if (Tcl_SetChannelOption (interp, outChan, TCL_TRANSLATION_OPT,
+                                  inChanTrans.string) != TCL_OK)
+            goto errorExit;
+        if (Tcl_SetChannelOption (interp, inChan, TCL_EOFCHAR_OPT,
+                                  inChanEOF.string) != TCL_OK)
+            goto errorExit;
+        if (Tcl_SetChannelOption (interp, outChan, TCL_EOFCHAR_OPT,
+                                  outChanEOF.string) != TCL_OK)
+            goto errorExit;
     }
 
     if (totalBytesRead < 0) {
         Tcl_SetErrno (saveErrno);
         Tcl_AppendResult (interp, "copyfile failed: ", Tcl_PosixError (interp),
                           (char *) NULL);
-        return TCL_ERROR;
+        goto errorExit;
     }
 
     /*
@@ -330,11 +337,23 @@ Tcl_CopyfileCmd (clientData, interp, argc, argv)
         sprintf (interp->result,
                  "premature EOF, %ld bytes expected, %ld bytes actually read",
                  totalBytesToRead, totalBytesRead);
-        return TCL_ERROR;
+        goto errorExit;
     }
 
     sprintf (interp->result, "%ld", totalBytesRead);
+    Tcl_DStringFree (&inChanTrans);
+    Tcl_DStringFree (&outChanTrans);
+    Tcl_DStringFree (&inChanEOF);
+    Tcl_DStringFree (&outChanEOF);
     return TCL_OK;
+
+  errorExit:
+    Tcl_DStringFree (&inChanTrans);
+    Tcl_DStringFree (&outChanTrans);
+    Tcl_DStringFree (&inChanEOF);
+    Tcl_DStringFree (&outChanEOF);
+
+    return TCL_ERROR;
 }
 
 /*-----------------------------------------------------------------------------
@@ -650,60 +669,6 @@ Tcl_LgetsCmd (notUsed, interp, argc, argv)
     return TCL_ERROR;
 }
 
-
-/*-----------------------------------------------------------------------------
- * Tcl_FrenameCmd --
- *     Implements the frename TCL command:
- *         frename oldPath newPath
- *
- * Results:
- *      Standard TCL result.
- *-----------------------------------------------------------------------------
- */
-int
-Tcl_FrenameCmd (clientData, interp, argc, argv)
-    ClientData  clientData;
-    Tcl_Interp *interp;
-    int         argc;
-    char      **argv;
-{
-    Tcl_DString pathBuf1, pathBuf2;
-    char *oldPath, *newPath;
-
-    if (argc != 3) {
-        Tcl_AppendResult (interp, tclXWrongArgs, argv [0], 
-                          " oldPath newPath", (char *) NULL);
-        return TCL_ERROR;
-    }
-
-    Tcl_DStringInit (&pathBuf1);
-    Tcl_DStringInit (&pathBuf2);
-    
-    oldPath = Tcl_TranslateFileName (interp, argv [1], &pathBuf1);
-    if (oldPath == NULL)
-        goto errorExit;
-
-    newPath = Tcl_TranslateFileName (interp, argv [2], &pathBuf2);
-    if (newPath == NULL)
-        goto errorExit;
-
-    if (rename (oldPath, newPath) != 0) {
-        Tcl_AppendResult (interp, "rename \"", argv [1], "\" to \"", argv [2],
-                          "\" failed: ", Tcl_PosixError (interp),
-                          (char *) NULL);
-        return TCL_ERROR;
-    }
-    
-    Tcl_DStringFree (&pathBuf1);
-    Tcl_DStringFree (&pathBuf2);
-    return TCL_OK;
-
-  errorExit:
-    Tcl_DStringFree (&pathBuf1);
-    Tcl_DStringFree (&pathBuf2);
-    return TCL_ERROR;
-}
-
 
 /*-----------------------------------------------------------------------------
  * TruncateByPath --
@@ -750,51 +715,6 @@ TruncateByPath (interp, filePath, newSize)
 }
 
 /*-----------------------------------------------------------------------------
- * TruncateByHandle --
- * 
- *  Truncate a file via a open file handle, if this is available on this
- * system.
- *
- * Parameters:
- *   o interp (I) - Error messages are returned in the interpreter.
- *   o fileHandle (I) - Path to file.
- *   o newSize (I) - Size to truncate the file to.
- * Returns:
- *   TCL_OK or TCL_ERROR.
- *-----------------------------------------------------------------------------
- */
-static int
-TruncateByHandle (interp, fileHandle, newSize)
-    Tcl_Interp  *interp;
-    char        *fileHandle;
-    off_t        newSize;
-{
-#if (!defined(NO_FTRUNCATE)) || defined(HAVE_CHSIZE)
-    int fileNum, stat;
-
-    fileNum = TclX_GetOpenFnum (interp, fileHandle, TCL_WRITABLE);
-    if (fileNum < 0)
-        return TCL_ERROR;
-
-#ifndef NO_FTRUNCATE
-    stat = ftruncate (fileNum, newSize);
-#else
-    stat = chsize (fileNum, newSize);
-#endif
-    if (stat != 0) {
-        Tcl_AppendResult (interp, fileHandle, ": ", Tcl_PosixError (interp),
-                          (char *) NULL);
-        return TCL_ERROR;
-    }
-    return TCL_OK;
-#else
-    Tcl_AppendResult (interp, FILE_ID_NOT_AVAIL, (char *) NULL);
-    return TCL_ERROR;
-#endif
-}
-
-/*-----------------------------------------------------------------------------
- *
  * Tcl_FtruncateCmd --
  *     Implements the Tcl ftruncate command:
  *     ftruncate [-fileid] file newsize
@@ -811,8 +731,9 @@ Tcl_FtruncateCmd (clientData, interp, argc, argv)
     int          argc;
     char       **argv;
 {
-    int    argIdx, fileIds;
-    off_t  newSize;
+    int argIdx, fileIds;
+    off_t newSize;
+    Tcl_Channel channel;
 
     fileIds = FALSE;
     for (argIdx = 1; (argIdx < argc) && (argv [argIdx] [0] == '-'); argIdx++) {
@@ -839,12 +760,45 @@ Tcl_FtruncateCmd (clientData, interp, argc, argv)
 
 
     if (fileIds) {
-        return TruncateByHandle (interp, argv [argIdx], newSize);
+        channel = TclX_GetOpenChannel (interp, argv [argIdx], 0);
+        if (channel == NULL)
+            return TCL_ERROR;
+        return TclXOSftruncate (interp, channel, newSize,
+                                "-fileid option");
     } else {
         return TruncateByPath (interp, argv [argIdx], newSize);
     }
 }
 
+/*-----------------------------------------------------------------------------
+ * ReadDirCallback --
+ *
+ *   Callback procedure for walking directories.
+ * Parameters:
+ *   o interp (I) - Interp is passed though.
+ *   o path (I) - Normalized path to directory.
+ *   o fileName (I) - Tcl normalized file name in directory.
+ *   o caseSensitive (I) - Are the file names case sensitive?  Always
+ *     TRUE on Unix.
+ *   o clientData (I) - Tcl_DString to append names to.
+ * Returns:
+ *   TCL_OK.
+ *-----------------------------------------------------------------------------
+ */
+static int
+ReadDirCallback (interp, path, fileName, caseSensitive, clientData)
+    Tcl_Interp  *interp;
+    char        *path;
+    char        *fileName;
+    int          caseSensitive;
+    ClientData   clientData;
+{
+    Tcl_DString *fileList = (Tcl_DString *) clientData;
+
+    Tcl_DStringAppendElement (fileList, fileName);
+    return TCL_OK;
+}
+
 /*-----------------------------------------------------------------------------
  * Tcl_ReaddirCmd --
  *     Implements the rename TCL command:
@@ -863,10 +817,8 @@ Tcl_ReaddirCmd (clientData, interp, argc, argv)
 {
     Tcl_DString pathBuf;
     char *dirPath;
-    TCLX_DIRHANDLE handle;
-    int hidden, status, caseSensitive;
+    int hidden, status;
     Tcl_DString fileList;
-    char *fileName;
     
     if ((argc < 2) || (argc > 3)) {
         Tcl_AppendResult (interp, tclXWrongArgs, argv [0], 
@@ -893,26 +845,18 @@ Tcl_ReaddirCmd (clientData, interp, argc, argv)
     if (dirPath == NULL)
         goto errorExit;
 
-    if (TclX_OSopendir (interp, dirPath, &handle, &caseSensitive) != TCL_OK)
+    status = TclXOSWalkDir (interp,
+                            dirPath,
+                            hidden,
+                            ReadDirCallback,
+                            (ClientData) &fileList);
+    if (status == TCL_ERROR)
         goto errorExit;
-    
-    while (TRUE) {
-        status = TclX_OSreaddir (interp, handle, hidden, &fileName);
-        if (status == TCL_ERROR)
-            goto errorExit2;
-        if (status == TCL_BREAK)
-            break;
-        Tcl_DStringAppendElement (&fileList, fileName);
-    }
-    if (TclX_OSclosedir (interp, handle) != TCL_OK)
-        goto errorExit2;
+
     Tcl_DStringFree (&pathBuf);
     Tcl_DStringResult (interp, &fileList);
     return TCL_OK;
 
-  errorExit2:
-    TclX_OSclosedir (NULL, handle);
-
   errorExit:
     Tcl_DStringFree (&pathBuf);
     Tcl_DStringFree (&fileList);
