@@ -13,7 +13,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXfilescan.c,v 7.0 1996/06/16 05:30:20 markd Exp $
+ * $Id: tclXfilescan.c,v 8.0 1996/11/21 00:24:05 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -44,6 +44,25 @@ typedef struct scanContext_t {
     int          fileOpen;
 } scanContext_t;
 
+/*
+ * Data kept on a specific scan.
+ */
+typedef struct {
+    int               storedLine;   /* Has the current line been stored in
+                                       matchInfo? */
+    scanContext_t    *contextPtr;   /* Current scan context. */
+    Tcl_Channel       channel;      /* The channel being scanned. */
+    char             *line;         /* The line from the file. */
+    off_t             offset;       /* The offset into the file. */
+    long              bytesRead;    /* Number of translated bytes read.*/
+    long              lineNum;      /* Current scanned line in the file. */
+    matchDef_t       *matchPtr;     /* The current match, or NULL for the
+                                       default. */
+    Tcl_SubMatchInfo  subMatchInfo; /* Information about the subexpressions
+                                       that matched, or NULL if this is the
+                                       default match. */
+} scanData_t;
+
 /*
  * Prototypes of internal functions.
  */
@@ -66,6 +85,18 @@ ScanContextCopyFile _ANSI_ARGS_((Tcl_Interp  *interp,
                                  char        *contextHandle,
                                  char        *fileHandle));
 
+static int
+Tcl_ScancontextCmd _ANSI_ARGS_((ClientData  clientData,
+                                Tcl_Interp *interp,
+                                int         argc,
+                                char      **argv));
+
+static int
+Tcl_ScanmatchCmd _ANSI_ARGS_((ClientData  clientData,
+                              Tcl_Interp *interp,
+                              int         argc,
+                              char      **argv));
+
 static void
 CopyFileCloseHandler _ANSI_ARGS_((ClientData clientData));
 
@@ -79,13 +110,7 @@ ClearCopyFile _ANSI_ARGS_((scanContext_t *contextPtr));
 
 static int
 SetMatchInfoVar _ANSI_ARGS_((Tcl_Interp       *interp,
-                             int              *storedLinePtr,
-                             scanContext_t    *contextPtr,
-                             Tcl_Channel       channel,
-                             char             *fileLine,
-                             long              scanLineNum,
-                             matchDef_t       *matchPtr,
-                             Tcl_SubMatchInfo  subMatchInfo));
+                             scanData_t       *scanData));
 
 static int
 ScanFile _ANSI_ARGS_((Tcl_Interp    *interp,
@@ -95,6 +120,12 @@ ScanFile _ANSI_ARGS_((Tcl_Interp    *interp,
 static void
 ScanFileCloseHandler _ANSI_ARGS_((ClientData clientData));
 
+static int
+Tcl_ScanfileCmd _ANSI_ARGS_((ClientData  clientData,
+                             Tcl_Interp *interp,
+                             int         argc,
+                             char      **argv));
+
 static void
 FileScanCleanUp _ANSI_ARGS_((ClientData  clientData,
                              Tcl_Interp *interp));
@@ -230,11 +261,11 @@ SetCopyFile (interp, contextPtr, fileHandle)
     if (contextPtr->copyFileChannel != NULL) {
         Tcl_DeleteCloseHandler (contextPtr->copyFileChannel,
                                 CopyFileCloseHandler,
-                                contextPtr);
+                                (ClientData) contextPtr);
     }
     Tcl_CreateCloseHandler (copyFileChannel,
                             CopyFileCloseHandler,
-                            contextPtr);
+                            (ClientData) contextPtr);
     contextPtr->copyFileChannel = copyFileChannel;
     return TCL_OK;
 }
@@ -253,7 +284,7 @@ ClearCopyFile (contextPtr)
     if (contextPtr->copyFileChannel != NULL) {
         Tcl_DeleteCloseHandler (contextPtr->copyFileChannel,
                                 CopyFileCloseHandler,
-                                contextPtr);
+                                (ClientData) contextPtr);
         contextPtr->copyFileChannel = NULL;
     }
 }
@@ -274,7 +305,6 @@ ScanContextCopyFile (interp, scanTablePtr, contextHandle, fileHandle)
     char        *fileHandle;
 {
     scanContext_t *contextPtr, **tableEntryPtr;
-    Tcl_Channel    copyFileChannel;
 
     tableEntryPtr = (scanContext_t **) Tcl_HandleXlate (interp,
                                                         scanTablePtr,
@@ -473,83 +503,78 @@ argError:
  * Parameters:
  *   o interp (O) - The Tcl interpreter to set the matchInfo variable in.
  *     Errors are returned in result.
- *   o storedLinePtr (I/O) - A flag that indicates if the current line has
+ *   o scanData (I/O) - Data about the current line being scanned.
  *     been stored.  Should be set to FALSE when a new line is read.
- *   o contextPtr (I) - The current scan context.
- *   o channel (I) - The file being scanned.
- *   o fileLine (I) - The current line.
- *   o scanLineNum (I) - Current scanned line in the file.
- *   o matchPtr (I) - The current match, or NULL fo the default.
- *   o subMatchInfo (I) - Information about the subexpressions that matched,
- *     or NULL if this is the default match.
  *-----------------------------------------------------------------------------
  */
 static int
-SetMatchInfoVar (interp, storedLinePtr, contextPtr, channel, fileLine,
-                 scanLineNum, matchPtr, subMatchInfo)
-    Tcl_Interp       *interp;
-    int              *storedLinePtr;
-    scanContext_t    *contextPtr;
-    Tcl_Channel       channel;
-    char             *fileLine;
-    long              scanLineNum;
-    matchDef_t       *matchPtr;
-    Tcl_SubMatchInfo  subMatchInfo;
+SetMatchInfoVar (interp, scanData)
+    Tcl_Interp *interp;
+    scanData_t *scanData;
 {
     static char *MATCHINFO = "matchInfo";
-    off_t matchOffset;
     int idx, start, end;
     char key [32], buf [32], *varPtr, holdChar;
 
     /*
      * Save information about the current line, if it hasn't been saved.
      */
-    if (!*storedLinePtr) {
-        *storedLinePtr = TRUE;
+    if (!scanData->storedLine) {
+        scanData->storedLine = TRUE;
 
         Tcl_UnsetVar (interp, MATCHINFO, 0);
         
-        matchOffset = Tcl_Tell (channel) - (strlen (fileLine) + 1);
-
-        if (Tcl_SetVar2 (interp, MATCHINFO, "line", fileLine, 
+        if (Tcl_SetVar2 (interp, MATCHINFO, "line", scanData->line, 
                          TCL_LEAVE_ERR_MSG) == NULL)
             return TCL_ERROR;
 
-        sprintf (buf, "%ld", matchOffset);
+        sprintf (buf, "%ld", scanData->offset);
         if (Tcl_SetVar2 (interp, MATCHINFO, "offset", buf,
                          TCL_LEAVE_ERR_MSG) == NULL)
             return TCL_ERROR;
 
-        sprintf (buf, "%ld", scanLineNum);
+#if 0
+        /*
+         * FIX: Don't expose till we decide on semantics: Should it include the
+         * current line?  All the pieces are here, include doc and tests, just
+         * disabled.
+         */
+        sprintf (buf, "%ld", scanData->bytesRead);
+        if (Tcl_SetVar2 (interp, MATCHINFO, "bytesread", buf,
+                         TCL_LEAVE_ERR_MSG) == NULL)
+            return TCL_ERROR;
+#endif
+
+        sprintf (buf, "%ld", scanData->lineNum);
         if (Tcl_SetVar2 (interp, MATCHINFO, "linenum", buf,
                          TCL_LEAVE_ERR_MSG) == NULL)
             return TCL_ERROR;
 
         if (Tcl_SetVar2 (interp, MATCHINFO, "context",
-                         contextPtr->contextHandle,
+                         scanData->contextPtr->contextHandle,
                          TCL_LEAVE_ERR_MSG) == NULL)
             return TCL_ERROR;
 
         if (Tcl_SetVar2 (interp, MATCHINFO, "handle", 
-                         Tcl_GetChannelName (channel),
+                         Tcl_GetChannelName (scanData->channel),
                          TCL_LEAVE_ERR_MSG) == NULL)
             return TCL_ERROR;
 
     }
 
-    if (contextPtr->copyFileChannel != NULL) {
+    if (scanData->contextPtr->copyFileChannel != NULL) {
         if (Tcl_SetVar2 (interp, MATCHINFO, "copyHandle", 
-                         Tcl_GetChannelName (contextPtr->copyFileChannel),
-                         TCL_LEAVE_ERR_MSG) == NULL)
+                Tcl_GetChannelName (scanData->contextPtr->copyFileChannel),
+                TCL_LEAVE_ERR_MSG) == NULL)
             return TCL_ERROR;
     }
 
-    if (subMatchInfo == NULL)
+    if (scanData->matchPtr == NULL)
         return TCL_OK;
 
-    for (idx = 0; idx < matchPtr->regExpInfo.numSubExprs; idx++) {
-        start = subMatchInfo [idx].start;
-        end = subMatchInfo [idx].end;
+    for (idx = 0; idx < scanData->matchPtr->regExpInfo.numSubExprs; idx++) {
+        start = scanData->subMatchInfo [idx].start;
+        end = scanData->subMatchInfo [idx].end;
 
         sprintf (key, "subindex%d", idx);
         sprintf (buf, "%d %d", start, end);
@@ -564,12 +589,12 @@ SetMatchInfoVar (interp, storedLinePtr, contextPtr, channel, fileLine,
                                   "",
                                   TCL_LEAVE_ERR_MSG);
         } else {
-            holdChar = fileLine [end + 1];
-            fileLine [end + 1] = '\0';
+            holdChar = scanData->line [end + 1];
+            scanData->line [end + 1] = '\0';
             varPtr = Tcl_SetVar2 (interp, "matchInfo", key,
-                                  fileLine + start,
+                                  scanData->line + start,
                                   TCL_LEAVE_ERR_MSG);
-            fileLine [end + 1] = holdChar;
+            scanData->line [end + 1] = holdChar;
         }
         if (varPtr == NULL)
             return TCL_ERROR;
@@ -590,26 +615,31 @@ ScanFile (interp, contextPtr, channel)
     Tcl_Channel    channel;
 {
     Tcl_DString buffer, lowerBuffer;
-    matchDef_t *matchPtr;
-    int result, matchedAtLeastOne, status, storedThisLine;
-    long scanLineNum = 0;
-    Tcl_SubMatchInfo subMatchInfo;
-
+    int result, matchedAtLeastOne;
+    scanData_t data;
+    
     if (contextPtr->matchListHead == NULL) {
         Tcl_AppendResult (interp, "no patterns in current scan context",
                           (char *) NULL);
         return TCL_ERROR;
     }
 
+    data.storedLine = FALSE;
+    data.contextPtr = contextPtr;
+    data.channel = channel;
+    data.bytesRead = 0;
+    data.lineNum = 0;
+    
     Tcl_DStringInit (&buffer);
     Tcl_DStringInit (&lowerBuffer);
 
     result = TCL_OK;
     while (TRUE) {
-        Tcl_DStringSetLength (&buffer, 0);
         if (!contextPtr->fileOpen)
             goto scanExit;  /* Closed by a callback */
 
+        data.offset = Tcl_Tell (channel);
+        Tcl_DStringSetLength (&buffer, 0);
         if (Tcl_Gets (channel, &buffer) < 0) {
             if (Tcl_Eof (channel) || Tcl_InputBlocked (channel))
                 goto scanExit;
@@ -617,41 +647,37 @@ ScanFile (interp, contextPtr, channel)
             result = TCL_ERROR;
             goto scanExit;
         }
-
-        scanLineNum++;
-        storedThisLine = FALSE;
+        data.line = buffer.string;
+        data.bytesRead += (buffer.length + 1);  /* Include EOLN */
+        data.lineNum++;
+        data.storedLine = FALSE;
         matchedAtLeastOne = FALSE;
 
         if (contextPtr->flags & CONTEXT_A_CASE_INSENSITIVE_FLAG) {
-            Tcl_DStringFree (&lowerBuffer);
+            Tcl_DStringSetLength (&lowerBuffer, 0);
             Tcl_DStringAppend (&lowerBuffer, buffer.string, -1);
             Tcl_DownShift (lowerBuffer.string, lowerBuffer.string);
         }
 
-        for (matchPtr = contextPtr->matchListHead; matchPtr != NULL; 
-                 matchPtr = matchPtr->nextMatchDefPtr) {
+        for (data.matchPtr = contextPtr->matchListHead; 
+             data.matchPtr != NULL; 
+             data.matchPtr = data.matchPtr->nextMatchDefPtr) {
 
             if (!TclX_RegExpExecute (interp,
-                                     &matchPtr->regExpInfo,
+                                     &(data.matchPtr->regExpInfo),
                                      buffer.string,
                                      lowerBuffer.string,
-                                     subMatchInfo))
+                                     data.subMatchInfo))
                 continue;  /* Try next match pattern */
 
             matchedAtLeastOne = TRUE;
 
-            result = SetMatchInfoVar(interp,
-                                     &storedThisLine,
-                                     contextPtr,
-                                     channel,
-                                     buffer.string,
-                                     scanLineNum,
-                                     matchPtr,
-                                     subMatchInfo);
+            result = SetMatchInfoVar (interp,
+                                      &data);
             if (result != TCL_OK)
                 goto scanExit;
 
-            result = Tcl_Eval(interp, matchPtr->command);
+            result = Tcl_Eval(interp, data.matchPtr->command);
             if (result == TCL_ERROR) {
                 Tcl_AddErrorInfo (interp, 
                     "\n    while executing a match command");
@@ -677,14 +703,9 @@ ScanFile (interp, contextPtr, channel)
          * Process default action if required.
          */
         if ((contextPtr->defaultAction != NULL) && (!matchedAtLeastOne)) {
+            data.matchPtr = NULL;
             result = SetMatchInfoVar(interp,
-                                     &storedThisLine,
-                                     contextPtr,
-                                     channel,
-                                     buffer.string,
-                                     scanLineNum,
-                                     NULL,
-                                     NULL);
+                                     &data);
             if (result != TCL_OK)
                 goto scanExit;
 
@@ -792,13 +813,13 @@ Tcl_ScanfileCmd (clientData, interp, argc, argv)
     contextPtr->fileOpen = TRUE;
     Tcl_CreateCloseHandler (channel,
                             ScanFileCloseHandler,
-                            contextPtr);
+                            (ClientData) contextPtr);
     status = ScanFile (interp,
                        contextPtr,
                        channel);
     Tcl_DeleteCloseHandler (channel,
                             ScanFileCloseHandler,
-                            contextPtr);
+                            (ClientData) contextPtr);
 
     /*
      * If we set the copyfile, disassociate it from the context.
@@ -864,10 +885,10 @@ Tcl_InitFilescan (interp)
      * Initialize the commands.
      */
     Tcl_CreateCommand (interp, "scanfile", Tcl_ScanfileCmd, 
-                       (ClientData) scanTablePtr, (void (*)()) NULL);
+                       (ClientData) scanTablePtr, (Tcl_CmdDeleteProc*) NULL);
     Tcl_CreateCommand (interp, "scanmatch", Tcl_ScanmatchCmd, 
-                       (ClientData) scanTablePtr, (void (*)()) NULL);
+                       (ClientData) scanTablePtr, (Tcl_CmdDeleteProc*) NULL);
     Tcl_CreateCommand (interp, "scancontext", Tcl_ScancontextCmd,
-                       (ClientData) scanTablePtr, (void (*)()) NULL);
+                       (ClientData) scanTablePtr, (Tcl_CmdDeleteProc*) NULL);
 }
 
