@@ -1,7 +1,7 @@
 /* 
  * tclXgeneral.c --
  *
- *      Contains general extensions to the basic TCL command set.
+ * A collection of general commands: echo, infox and loop.
  *-----------------------------------------------------------------------------
  * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
  *
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXgeneral.c,v 7.0 1996/06/16 05:30:25 markd Exp $
+ * $Id: tclXgeneral.c,v 8.0 1996/11/21 00:24:07 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -131,6 +131,8 @@ Tcl_InfoxCmd (clientData, interp, argc, argv)
 {
     char numBuf [32];
 
+    /* FIX: Need a way to get the have_ functionallity from the OS dependent
+       code */
     if (argc != 2) {
         Tcl_AppendResult (interp, tclXWrongArgs, argv [0], 
                           " option", (char *) NULL);
@@ -164,11 +166,7 @@ Tcl_InfoxCmd (clientData, interp, argc, argv)
         return TCL_OK;
     }
     if (STREQU ("have_flock", argv [1])) {
-#       ifdef F_SETLKW
-        interp->result = "1";
-#       else
-        interp->result = "0";
-#       endif        
+        interp->result = TclXOSHaveFlock () ? "1" : "0";
         return TCL_OK;
     }
     if (STREQU ("have_fsync", argv [1])) {
@@ -208,6 +206,14 @@ Tcl_InfoxCmd (clientData, interp, argc, argv)
         interp->result = "1";
 #       else
         interp->result = "0";
+#       endif        
+        return TCL_OK;
+    }
+    if (STREQU ("have_symlink", argv [1])) {
+#       ifdef S_IFLNK
+        interp->result = "1";
+#       else
+        interp->result = "0";
 #       endif        
         return TCL_OK;
     }
@@ -244,7 +250,7 @@ Tcl_InfoxCmd (clientData, interp, argc, argv)
                       "\" expect one of: version, patchlevel, ",
                       "have_fchown, have_fchmod, have_flock, ",
                       "have_fsync, have_ftruncate, have_msgcats, ",
-                      "have_truncate, have_posix_signals, ",
+                      "have_symlink, have_truncate, have_posix_signals, ",
                       "have_waitpid, ",
                       "appname, applongname, appversion, or apppatchlevel",
                       (char *) NULL);
@@ -318,7 +324,7 @@ Tcl_LoopCmd (dummy, interp, argc, argv)
     /*
      * Set variable to its final value.
      */
-    sprintf (itxt,"%ld",i);
+    sprintf (itxt, "%ld" ,i);
     if (Tcl_SetVar (interp, argv [1], itxt, TCL_LEAVE_ERR_MSG) == NULL)
         return TCL_ERROR;
 
