@@ -13,7 +13,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXsignal.c,v 7.0 1996/06/16 05:30:54 markd Exp $
+ * $Id: tclXsignal.c,v 8.0 1996/11/21 00:24:18 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -251,6 +251,11 @@ GetSignalState _ANSI_ARGS_((int              signalNum,
 static int
 SetSignalState _ANSI_ARGS_((int             signalNum,
                             signalProcPtr_t sigFunc));
+static int
+Tcl_SignalCmd _ANSI_ARGS_((ClientData  clientData,
+                           Tcl_Interp *interp,
+                           int         argc,
+                           char      **argv));
 
 static int
 BlockSignals _ANSI_ARGS_((Tcl_Interp    *interp,
@@ -330,6 +335,11 @@ SetSignalStates _ANSI_ARGS_((Tcl_Interp *interp,
 static void
 SignalCmdCleanUp _ANSI_ARGS_((ClientData  clientData,
                               Tcl_Interp *interp));
+static int
+Tcl_KillCmd _ANSI_ARGS_((ClientData  clientData,
+                         Tcl_Interp *interp,
+                         int         argc,
+                         char      **argv));
 
 
 /*-----------------------------------------------------------------------------
@@ -1509,7 +1519,7 @@ Tcl_SignalCmd (clientData, interp, argc, argv)
  *  Standard TCL results, may return the UNIX system error message.
  *-----------------------------------------------------------------------------
  */
-int
+static int
 Tcl_KillCmd (clientData, interp, argc, argv)
     ClientData  clientData;
     Tcl_Interp *interp;
@@ -1563,7 +1573,7 @@ Tcl_KillCmd (clientData, interp, argc, argv)
         if (pgroup)
             procId = -procId;
 
-        if (TclX_OSkill (interp, procId, signalNum, argv [0]) != TCL_OK)
+        if (TclXOSkill (interp, procId, signalNum, argv [0]) != TCL_OK)
             goto errorExit;
     }
 
@@ -1605,6 +1615,7 @@ SignalCmdCleanUp (clientData, interp)
     if (idx == numInterps)
         panic ("signal interp lost");
 
+    Tcl_AsyncDelete(interpTable[idx].handler);
     interpTable [idx] = interpTable [--numInterps];
 
     /*
@@ -1717,7 +1728,7 @@ Tcl_InitSignalHandling (interp)
     Tcl_CallWhenDeleted (interp, SignalCmdCleanUp, (ClientData) NULL);
 
     Tcl_CreateCommand (interp, "signal", Tcl_SignalCmd,
-                       (ClientData) NULL, (void (*)()) NULL);
+                       (ClientData) NULL, (Tcl_CmdDeleteProc*) NULL);
     Tcl_CreateCommand (interp, "kill", Tcl_KillCmd,
-                       (ClientData) NULL, (void (*)()) NULL);
+                       (ClientData) NULL, (Tcl_CmdDeleteProc*) NULL);
 }
