@@ -3,7 +3,7 @@
  *
  * Utility functions for Extended Tcl.
  *-----------------------------------------------------------------------------
- * Copyright 1991-1999 Karl Lehenbauer and Mark Diekhans.
+ * Copyright 1991-1996 Karl Lehenbauer and Mark Diekhans.
  *
  * Permission to use, copy, modify, and distribute this software and its
  * documentation for any purpose and without fee is hereby granted, provided
@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXutil.c,v 8.28 1999/03/31 06:37:47 markd Exp $
+ * $Id: tclXutil.c,v 8.0 1996/11/21 00:24:21 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -26,6 +26,9 @@
 /*
  * Prototypes of internal functions.
  */
+static int
+ReturnOverflow _ANSI_ARGS_((Tcl_Interp *interp));
+
 static int
 CallEvalErrorHandler _ANSI_ARGS_((Tcl_Interp  *interp));
 
@@ -35,25 +38,42 @@ ParseTranslationOption _ANSI_ARGS_((char *strValue));
 static char *
 FormatTranslationOption _ANSI_ARGS_((int value));
 
-static char *ERRORINFO = "errorInfo";
-static char *ERRORCODE = "errorCode";
-
 /*
  * Used to return argument messages by most commands.
  */
 char *tclXWrongArgs = "wrong # args: ";
 
-
 /*-----------------------------------------------------------------------------
- * TclX_StrToInt --
- *      Convert an Ascii string to an number of the specified base.
+ * ReturnOverflow --
+ *    
+ *   Return an error message about an numeric overflow.
  *
  * Parameters:
- *   o string - String containing a number.
- *   o base - The base to use for the number 8, 10 or 16 or zero to decide
+ *   o interp (O) - Interpreter to set the error message in.
+ * Returns:
+ *   TCL_ERROR;
+ *-----------------------------------------------------------------------------
+ */
+static int
+ReturnOverflow (interp)
+    Tcl_Interp *interp;
+{
+    interp->result = "integer value too large to represent";
+    Tcl_SetErrorCode(interp, "ARITH", "IOVERFLOW",
+                     interp->result, (char *) NULL);
+    return TCL_ERROR;
+}
+
+/*-----------------------------------------------------------------------------
+ * Tcl_StrToLong --
+ *      Convert an Ascii string to an long number of the specified base.
+ *
+ * Parameters:
+ *   o string (I) - String containing a number.
+ *   o base (I) - The base to use for the number 8, 10 or 16 or zero to decide
  *     based on the leading characters of the number.  Zero to let the number
  *     determine the base.
- *   o intPtr - Place to return the converted number.  Will be 
+ *   o longPtr (O) - Place to return the converted number.  Will be 
  *     unchanged if there is an error.
  *
  * Returns:
@@ -61,7 +81,67 @@ char *tclXWrongArgs = "wrong # args: ";
  *-----------------------------------------------------------------------------
  */
 int
-TclX_StrToInt (string, base, intPtr)
+Tcl_StrToLong (string, base, longPtr)
+    CONST char *string;
+    int         base;
+    long       *longPtr;
+{
+    char *end, *p;
+    long  i;
+
+    /*
+     * Note: use strtoul instead of strtol for integer conversions
+     * to allow full-size unsigned numbers, but don't depend on strtoul
+     * to handle sign characters;  it won't in some implementations.
+     */
+
+    errno = 0;
+    for (p = (char *) string; isspace(UCHAR(*p)); p++) {
+        /* Empty loop body. */
+    }
+    if (*p == '-') {
+        p++;
+        i = -(long) strtoul(p, &end, base);
+    } else if (*p == '+') {
+        p++;
+        i = strtoul(p, &end, base);
+    } else {
+        i = strtoul(p, &end, base);
+    }
+    if (end == p) {
+        return FALSE;
+    }
+    if (errno == ERANGE) {
+        return FALSE;
+    }
+    while ((*end != '\0') && isspace(UCHAR(*end))) {
+        end++;
+    }
+    if (*end != '\0') {
+        return FALSE;
+    }
+    *longPtr = i;
+    return TRUE;
+}
+
+/*-----------------------------------------------------------------------------
+ * Tcl_StrToInt --
+ *      Convert an Ascii string to an number of the specified base.
+ *
+ * Parameters:
+ *   o string (I) - String containing a number.
+ *   o base (I) - The base to use for the number 8, 10 or 16 or zero to decide
+ *     based on the leading characters of the number.  Zero to let the number
+ *     determine the base.
+ *   o intPtr (O) - Place to return the converted number.  Will be 
+ *     unchanged if there is an error.
+ *
+ * Returns:
+ *      Returns 1 if the string was a valid number, 0 invalid.
+ *-----------------------------------------------------------------------------
+ */
+int
+Tcl_StrToInt (string, base, intPtr)
     CONST char *string;
     int         base;
     int        *intPtr;
@@ -105,15 +185,15 @@ TclX_StrToInt (string, base, intPtr)
 }
 
 /*-----------------------------------------------------------------------------
- * TclX_StrToUnsigned --
+ * Tcl_StrToUnsigned --
  *      Convert an Ascii string to an unsigned int of the specified base.
  *
  * Parameters:
- *   o string - String containing a number.
- *   o base - The base to use for the number 8, 10 or 16 or zero to decide
+ *   o string (I) - String containing a number.
+ *   o base (I) - The base to use for the number 8, 10 or 16 or zero to decide
  *     based on the leading characters of the number.  Zero to let the number
  *     determine the base.
- *   o unsignedPtr - Place to return the converted number.  Will be 
+ *   o unsignedPtr (O) - Place to return the converted number.  Will be 
  *     unchanged if there is an error.
  *
  * Returns:
@@ -121,7 +201,7 @@ TclX_StrToInt (string, base, intPtr)
  *-----------------------------------------------------------------------------
  */
 int
-TclX_StrToUnsigned (string, base, unsignedPtr)
+Tcl_StrToUnsigned (string, base, unsignedPtr)
     CONST char *string;
     int         base;
     unsigned   *unsignedPtr;
@@ -151,15 +231,46 @@ TclX_StrToUnsigned (string, base, unsignedPtr)
 }
 
 /*-----------------------------------------------------------------------------
- * TclX_StrToOffset --
+ * Tcl_StrToDouble --
+ *   Convert a string to a double percision floating point number.
+ *
+ * Parameters:
+ *   string (I) - Buffer containing double value to convert.
+ *   doublePtr (O) - The convert floating point number.
+ * Returns:
+ *   TRUE if the number is ok, FALSE if it is illegal.
+ *-----------------------------------------------------------------------------
+ */
+int
+Tcl_StrToDouble (string, doublePtr)
+    CONST char *string;
+    double     *doublePtr;
+{
+    char   *end, *p;
+    double  i;
+
+    errno = 0;
+    for (p = (char *) string; isspace(UCHAR(*p)); p++) {
+        /* Empty loop body. */
+    }
+    i = strtod (string, &end);
+    if (end == p) {
+        return FALSE;
+    }
+    *doublePtr = i;
+    return TRUE;
+}
+
+/*-----------------------------------------------------------------------------
+ * Tcl_StrToOffset --
  *      Convert an Ascii string to an off_t number of the specified base.
  *
  * Parameters:
- *   o string - String containing a number.
- *   o base - The base to use for the number 8, 10 or 16 or zero to decide
+ *   o string (I) - String containing a number.
+ *   o base (I) - The base to use for the number 8, 10 or 16 or zero to decide
  *     based on the leading characters of the number.  Zero to let the number
  *     determine the base.
- *   o offsetPtr - Place to return the converted number.  Will be 
+ *   o offsetPtr (O) - Place to return the converted number.  Will be 
  *     unchanged if there is an error.
  *
  * Returns:
@@ -167,7 +278,7 @@ TclX_StrToUnsigned (string, base, unsignedPtr)
  *-----------------------------------------------------------------------------
  */
 int
-TclX_StrToOffset (string, base, offsetPtr)
+Tcl_StrToOffset (string, base, offsetPtr)
     CONST char *string;
     int         base;
     off_t      *offsetPtr;
@@ -211,25 +322,24 @@ TclX_StrToOffset (string, base, offsetPtr)
 }
 
 /*-----------------------------------------------------------------------------
- * TclX_DownShift --
+ * Tcl_DownShift --
  *     Utility procedure to down-shift a string.  It is written in such
  *     a way as that the target string maybe the same as the source string.
  *
  * Parameters:
- *   o targetStr - String to store the down-shifted string in.  Must
+ *   o targetStr (I) - String to store the down-shifted string in.  Must
  *     have enough space allocated to store the string.  If NULL is specified,
  *     then the string will be dynamicly allocated and returned as the
  *     result of the function. May also be the same as the source string to
  *     shift in place.
- *   o sourceStr - The string to down-shift.
+ *   o sourceStr (I) - The string to down-shift.
  *
  * Returns:
  *   A pointer to the down-shifted string
- * FIX: Make object based interface.
  *-----------------------------------------------------------------------------
  */
 char *
-TclX_DownShift (targetStr, sourceStr)
+Tcl_DownShift (targetStr, sourceStr)
     char       *targetStr;
     CONST char *sourceStr;
 {
@@ -248,25 +358,23 @@ TclX_DownShift (targetStr, sourceStr)
 }
 
 /*-----------------------------------------------------------------------------
- * TclX_UpShift --
+ * Tcl_UpShift --
  *     Utility procedure to up-shift a string.
  *
  * Parameters:
- *   o targetStr - String to store the up-shifted string in.  Must
+ *   o targetStr (I) - String to store the up-shifted string in.  Must
  *     have enough space allocated to store the string.  If NULL is specified,
  *     then the string will be dynamicly allocated and returned as the
  *     result of the function. May also be the same as the source string to
  *     shift in place.
- *   o sourceStr - The string to up-shift.
+ *   o sourceStr (I) - The string to up-shift.
  *
  * Returns:
- *   A pointer to the up-shifted string.
- * FIX: Get strcasecmp and replace this with it.
- * FIX: Make object based interface
+ *   A pointer to the up-shifted string
  *-----------------------------------------------------------------------------
  */
 char *
-TclX_UpShift (targetStr, sourceStr)
+Tcl_UpShift (targetStr, sourceStr)
     char       *targetStr;
     CONST char *sourceStr;
 {
@@ -285,111 +393,213 @@ TclX_UpShift (targetStr, sourceStr)
 }
 
 /*-----------------------------------------------------------------------------
- * TclX_GetOffsetFromObj --
- *   Get the value of an integer objects as an unsigned.
+ * Tcl_GetLong --
+ *
+ *      Given a string, produce the corresponding long value.
+ *
+ * Results:
+ *      The return value is normally TCL_OK;  in this case *longPtr
+ *      will be set to the integer value equivalent to string.  If
+ *      string is improperly formed then TCL_ERROR is returned and
+ *      an error message will be left in interp->result.
+ *
+ * Side effects:
+ *      None.
  *-----------------------------------------------------------------------------
  */
 int
-TclX_GetUnsignedFromObj (interp, objPtr, valuePtr)
+Tcl_GetLong(interp, string, longPtr)
     Tcl_Interp *interp;
-    Tcl_Obj    *objPtr;
-    unsigned   *valuePtr;
+    CONST char *string;
+    long       *longPtr;
 {
-    int intValue;
-    
-    if (Tcl_GetIntFromObj (interp, objPtr, &intValue) != TCL_OK)
-        return TCL_ERROR;
-    if (intValue < 0) {
-        TclX_AppendObjResult (interp, "expected unsigned integer, got \"",
-                              Tcl_GetStringFromObj (objPtr, NULL),
-                              "\"", (char *) NULL);
+    char *end, *p;
+    long i;
+
+    errno = 0;
+    for (p = (char *) string; isspace(UCHAR(*p)); p++) {
+        /* Empty loop body. */
+    }
+    i = strtol(p, &end, 0);
+    if (end == p) {
+        badInteger:
+        Tcl_AppendResult(interp, "expected integer but got \"", string,
+                "\"", (char *) NULL);
         return TCL_ERROR;
     }
-    *valuePtr = intValue;
+    if (errno == ERANGE) {
+        return ReturnOverflow (interp);
+    }
+    while ((*end != '\0') && isspace(UCHAR(*end))) {
+        end++;
+    }
+    if (*end != '\0') {
+        goto badInteger;
+    }
+    *longPtr = i;
     return TCL_OK;
 }
 
 /*-----------------------------------------------------------------------------
- * TclX_GetOffsetFromObj --
- *   Get the value of an integer objects as an off_t.
+ * Tcl_GetUnsigned --
+ *
+ *      Given a string, produce the corresponding unsigned integer value.
+ *
+ * Results:
+ *      The return value is normally TCL_OK;  in this case *unsignedPtr
+ *      will be set to the integer value equivalent to string.  If
+ *      string is improperly formed then TCL_ERROR is returned and
+ *      an error message will be left in interp->result.
+ *
+ * Side effects:
+ *      None.
  *-----------------------------------------------------------------------------
  */
 int
-TclX_GetOffsetFromObj (interp, objPtr, offsetPtr)
+Tcl_GetUnsigned(interp, string, unsignedPtr)
     Tcl_Interp *interp;
-    Tcl_Obj    *objPtr;
-    off_t      *offsetPtr;
+    CONST char *string;
+    unsigned   *unsignedPtr;
 {
-    int intOff;
-    
-    if (Tcl_GetIntFromObj (interp, objPtr, &intOff) != TCL_OK)
-        return TCL_ERROR;
-    *offsetPtr = intOff;
+    char          *end, *p;
+    unsigned long  i;
+
+    /*
+     * Since some strtoul functions don't detect negative numbers, check
+     * in advance.
+     */
+    errno = 0;
+    for (p = (char *) string; isspace(UCHAR(*p)); p++) {
+        /* Empty loop body. */
+    }
+    if (*p == '-')
+        goto badUnsigned;
+    if (*p == '+') {
+        p++;
+    }
+    i = strtoul(p, &end, 0);
+    if (end == p) {
+        goto badUnsigned;
+    }
+    if (errno == ERANGE) {
+        return ReturnOverflow (interp);
+    }
+    while ((*end != '\0') && isspace(UCHAR(*end))) {
+        end++;
+    }
+    if (*end != '\0') {
+        goto badUnsigned;
+    }
+
+    *unsignedPtr = i;
     return TCL_OK;
+
+  badUnsigned:
+    Tcl_AppendResult (interp, "expected unsigned integer but got \"", 
+                      string, "\"", (char *) NULL);
+    return TCL_ERROR;
 }
 
 /*-----------------------------------------------------------------------------
- * TclX_RelativeExpr --
+ * Tcl_GetOffset --
+ *
+ *      Given a string, produce the corresponding off_t value.
+ *
+ * Results:
+ *      The return value is normally TCL_OK;  in this case *timepPtr
+ *      will be set to the integer value equivalent to string.  If
+ *      string is improperly formed then TCL_ERROR is returned and
+ *      an error message will be left in interp->result.
+ *
+ * Side effects:
+ *      None.
+ *-----------------------------------------------------------------------------
+ */
+int
+Tcl_GetOffset(interp, string, offsetPtr)
+    Tcl_Interp *interp;
+    CONST char *string;
+    off_t      *offsetPtr;
+{
+    char *end, *p;
+    long i;
+
+    errno = 0;
+    for (p = (char *) string; isspace(UCHAR(*p)); p++) {
+        /* Empty loop body. */
+    }
+    i = strtol(p, &end, 0);
+    if (end == p) {
+        goto badOffset;
+    }
+    if (errno == ERANGE) {
+        return ReturnOverflow (interp);
+    }
+    while ((*end != '\0') && isspace(UCHAR(*end))) {
+        end++;
+    }
+    if (*end != '\0') {
+        goto badOffset;
+    }
+    *offsetPtr = (off_t) i;
+    if (*offsetPtr != i)
+        goto badOffset;
+    return TCL_OK;
+
+  badOffset:
+    Tcl_AppendResult (interp, "expected integer offset but got \"", 
+                      string, "\"", (char *) NULL);
+    return TCL_ERROR;
+}
+
+/*-----------------------------------------------------------------------------
+ * Tcl_RelativeExpr --
  *
  *    Evaluate an expression that may start with the magic words "end" or
  * "len".  These strings are replaced with either the end offset or the
  * length that is passed in.
  *
  * Parameters:
- *   o interp - A pointer to the interpreter.
- *   o exprPtr - Object with expression to evaluate.
- *   o stringLen - The length of the string or list.
- *   o exprResultPtr - The result of the expression is returned here.
+ *   o interp (I) - A pointer to the interpreter.
+ *   o cstringExpr (I) - The expression to evaludate.
+ *   o stringLen (I) - The length of the string.
+ *   o exprResultPtr (O) - The result of the expression is returned here.
  * Returns:
  *   TCL_OK or TCL_ERROR.
  *-----------------------------------------------------------------------------
  */
 int
-TclX_RelativeExpr (interp, exprPtr, stringLen, exprResultPtr)
+Tcl_RelativeExpr (interp, cstringExpr, stringLen, exprResultPtr)
     Tcl_Interp  *interp;
-    Tcl_Obj     *exprPtr;
-    int          stringLen;
-    int         *exprResultPtr;
+    char        *cstringExpr;
+    long         stringLen;
+    long        *exprResultPtr;
 {
-    char *exprStr, *buf;
-    int exprLen, exprStrLen, result;
-    long longResult;
-    char staticBuf [32];
+    
+    char *buf;
+    int   exprLen, result;
+    char  staticBuf [64];
 
-    if (exprPtr->typePtr == Tcl_GetObjType ("int")) {
-        if (Tcl_GetIntFromObj (interp, exprPtr, exprResultPtr) != TCL_OK)
-            return TCL_ERROR;
-        return TCL_OK;
+    if (!(STRNEQU (cstringExpr, "end", 3) ||
+          STRNEQU (cstringExpr, "len", 3))) {
+        return Tcl_ExprLong (interp, cstringExpr, exprResultPtr);
     }
 
-    exprStr = Tcl_GetStringFromObj (exprPtr, &exprStrLen);
-
-    if (!(STRNEQU (exprStr, "end", 3) ||
-          STRNEQU (exprStr, "len", 3))) {
-        if (Tcl_ExprLong (interp, exprStr, &longResult) != TCL_OK) {
-            return TCL_ERROR;
-        }
-        *exprResultPtr = longResult;
-        return TCL_OK;
-    }
-
-    sprintf (staticBuf, "%d",
-             stringLen - ((exprStr [0] == 'e') ? 1 : 0));
-    exprLen = strlen (staticBuf) + exprStrLen - 2;
+    sprintf (staticBuf, "%ld",
+             stringLen - ((cstringExpr [0] == 'e') ? 1 : 0));
+    exprLen = strlen (staticBuf) + strlen (cstringExpr) - 2;
 
     buf = staticBuf;
     if (exprLen > sizeof (staticBuf)) {
         buf = (char *) ckalloc (exprLen);
         strcpy (buf, staticBuf);
     }
-    strcat (buf, exprStr + 3);
+    strcat (buf, cstringExpr + 3);
 
-    result = Tcl_ExprLong (interp, buf, &longResult);
+    result = Tcl_ExprLong (interp, buf, exprResultPtr);
 
     if (buf != staticBuf)
         ckfree (buf);
-    if (result == TCL_OK)
-        *exprResultPtr = longResult;
     return result;
 }
 
@@ -399,79 +609,35 @@ TclX_RelativeExpr (interp, exprPtr, stringLen, exprResultPtr)
  *    Convert a file handle to a channel with error checking.
  *
  * Parameters:
- *   o interp - Current interpreter.
- *   o handle - The file handle to convert.
- *   o chanAccess - TCL_READABLE and/or TCL_WRITABLE, both or zero for no
- *     checking.
+ *   o interp (I) - Current interpreter.
+ *   o handle (I) - The file handle to convert.
+ *   o direction (I) - TCL_READABLE or TCL_WRITABLE, or zero.  If zero, then
+ *     return the first of the read and write numbers.
  * Returns:
  *   A the channel or NULL if an error occured.
  *-----------------------------------------------------------------------------
  */
 Tcl_Channel
-TclX_GetOpenChannel (interp, handle, chanAccess)
+TclX_GetOpenChannel (interp, handle, direction)
     Tcl_Interp *interp;
     char       *handle;
-    int         chanAccess;
+    int         direction;
 {
     Tcl_Channel chan;
     int mode;
 
-    /*FIX: Delete when all converted to GetOpenChanelObj */
     chan = Tcl_GetChannel (interp, handle, &mode);
     if (chan == (Tcl_Channel) NULL) {
+	return NULL;
+    }
+    if ((direction & TCL_READABLE) && ((mode & TCL_READABLE) == 0)) {
+        Tcl_AppendResult(interp, "channel \"", handle,
+                "\" wasn't opened for reading", (char *) NULL);
         return NULL;
     }
-    if ((chanAccess & TCL_READABLE) && ((mode & TCL_READABLE) == 0)) {
-        TclX_AppendObjResult(interp, "channel \"", handle,
-                             "\" wasn't opened for reading", (char *) NULL);
-        return NULL;
-    }
-    if ((chanAccess & TCL_WRITABLE) && ((mode & TCL_WRITABLE) == 0)) {
-        TclX_AppendObjResult(interp, "channel \"", handle,
-                             "\" wasn't opened for writing", (char *) NULL);
-        return NULL;
-    }
-
-    return chan;
-}
-
-/*-----------------------------------------------------------------------------
- * TclX_GetOpenChannelObj --
- *
- *    Convert a file handle to a channel with error checking.
- *
- * Parameters:
- *   o interp    - Current interpreter.
- *   o handleObj - The file handle object to convert.
- *   o chanAccess - TCL_READABLE and/or TCL_WRITABLE, both or zero for no
- *     checking.
- * Returns:
- *   A the channel or NULL if an error occured.
- *-----------------------------------------------------------------------------
- */
-Tcl_Channel
-TclX_GetOpenChannelObj (interp, handleObj, chanAccess)
-    Tcl_Interp *interp;
-    Tcl_Obj    *handleObj;
-    int         chanAccess;
-{
-    Tcl_Channel  chan;
-    int          mode;
-    char        *handle;
-
-    handle = Tcl_GetStringFromObj (handleObj, NULL);
-    chan = Tcl_GetChannel (interp, handle, &mode);
-    if (chan == (Tcl_Channel) NULL) {
-        return NULL;
-    }
-    if ((chanAccess & TCL_READABLE) && ((mode & TCL_READABLE) == 0)) {
-        TclX_AppendObjResult (interp, "channel \"", handle,
-                              "\" wasn't opened for reading", (char *) NULL);
-        return NULL;
-    }
-    if ((chanAccess & TCL_WRITABLE) && ((mode & TCL_WRITABLE) == 0)) {
-        TclX_AppendObjResult (interp, "channel \"", handle,
-                              "\" wasn't opened for writing", (char *) NULL);
+    if ((direction & TCL_WRITABLE) && ((mode & TCL_WRITABLE) == 0)) {
+        Tcl_AppendResult(interp, "channel \"", handle,
+                "\" wasn't opened for writing", (char *) NULL);
         return NULL;
     }
 
@@ -485,10 +651,10 @@ TclX_GetOpenChannelObj (interp, handleObj, chanAccess)
  * it the result of the failed command.
  *
  * Parameters:
- *   o interp - A pointer to the interpreter.
+ *   o interp (I) - A pointer to the interpreter.
  * Returns:
  *   The Tcl result code from the handler.  TCL_ERROR is returned and
- * result unchanged if no handler is available.
+ * result unchanged if not handler is available.
  *-----------------------------------------------------------------------------
  */
 static int
@@ -497,39 +663,38 @@ CallEvalErrorHandler (interp)
 {
     static char *ERROR_HANDLER = "tclx_errorHandler";
     Tcl_CmdInfo cmdInfo;
-    Tcl_Obj *errorHandler;
-    Tcl_Obj *command;
+    Tcl_DString command;
     int result;
 
-
     /*
      * Check if the tclx_errorHandler function exists.  For backwards
      * compatibility with TclX 7.4 we check to see if there is a variable
-     * by the same name holding the name of a procedure.  Build up the command
+     * by the same name holding the name of a procedrure.  Build up the command
      * based on what we found.  The variable functionality is deprectated and
-     * should be removed eventually. FIX: Delete.
+     * should be removed eventually. ????
      */
     if (!Tcl_GetCommandInfo (interp, ERROR_HANDLER, &cmdInfo)) {
-        errorHandler = Tcl_GetVar2Ex(interp, ERROR_HANDLER, NULL,
-                                     TCL_GLOBAL_ONLY);
+        char *errorHandler;
+
+        errorHandler = Tcl_GetVar (interp, ERROR_HANDLER, TCL_GLOBAL_ONLY);
         if (errorHandler == NULL)
             return TCL_ERROR;  /* No handler specified */
+        Tcl_DStringInit (&command);
+        Tcl_DStringAppendElement (&command, errorHandler);
     } else {
-        errorHandler = Tcl_NewStringObj (ERROR_HANDLER, -1);
+        Tcl_DStringInit (&command);
+        Tcl_DStringAppendElement (&command, ERROR_HANDLER);
     }
-    command = Tcl_NewListObj (0, NULL);
-    Tcl_IncrRefCount (command);
-    Tcl_ListObjAppendElement (NULL, command, errorHandler);
-    Tcl_ListObjAppendElement (NULL, command,
-                              Tcl_GetObjResult (interp));
-                              
-    result = Tcl_EvalObjEx (interp, command, TCL_EVAL_GLOBAL);
+
+    Tcl_DStringAppendElement (&command, interp->result);
+
+    result = Tcl_GlobalEval (interp, Tcl_DStringValue (&command));
     if (result == TCL_ERROR) {
         Tcl_AddErrorInfo (interp,
                           "\n    (while processing tclx_errorHandler)");
     }
 
-    Tcl_DecrRefCount (command);
+    Tcl_DStringFree (&command);
     return result;
 }
 
@@ -539,14 +704,14 @@ CallEvalErrorHandler (interp)
  *   Evaluate a Tcl command string with various options.
  *
  * Parameters:
- *   o interp - A pointer to the interpreter.
- *   o options - Options controling the evaluation:
+ *   o interp (I) - A pointer to the interpreter.
+ *   o options (I) - Options controling the evaluation:
  *     o TCLX_EVAL_GLOBAL - Evaulate in the global context.
  *     o TCLX_EVAL_FILE - Treat the string as the name of a file to eval.
  *     o TCLX_EVAL_ERR_HANDLER - Call the user-specified error callback 
  *       specified in the global variable tclx_errorHandler if an error
  *       occurs.
- *   o string - The command or name of file to evaluate.
+ *   o string (I) - The command or name of file to evaluate.
  * Returns:
  *   The Tcl result code.
  *-----------------------------------------------------------------------------
@@ -558,7 +723,7 @@ TclX_Eval (interp, options, string)
     char        *string;
 {
     Interp      *iPtr = (Interp *) interp;
-    CallFrame   *savedVarFramePtr = NULL;
+    CallFrame   *savedVarFramePtr;
     int          result;
 
     if (options & TCLX_EVAL_GLOBAL) {
@@ -588,9 +753,9 @@ TclX_Eval (interp, options, string)
  *   Evaluate a Tcl command string with various options.
  *
  * Parameters:
- *   o interp - A pointer to the interpreter.
- *   o options - Options controling the evaluation, see TclX_Eval.
- *   o str, ... - String arguments, terminated by a NULL.  They will
+ *   o interp (I) - A pointer to the interpreter.
+ *   o options (I) - Options controling the evaluation, see TclX_Eval.
+ *   o str, ... (I) - String arguments, terminated by a NULL.  They will
  *     be concatenated together to form a single string.
  *-----------------------------------------------------------------------------
  */
@@ -629,8 +794,8 @@ TclX_VarEval TCL_VARARGS_DEF(Tcl_Interp *, arg1)
  *   Write a string to a channel.
  *
  * Parameters:
- *   o channel - Channel to write to.
- *   o str - The string to write.
+ *   o channel (I) - Channel to write to.
+ *   o str (I) - The string to write.
  * Returns:
  *   Same as for Tcl_Write, -1 is an error.
  *-----------------------------------------------------------------------------
@@ -650,7 +815,7 @@ TclX_WriteStr (channel, str)
  * direction.
  *
  * Parameters:
- *   o strValue - Channel translation value.
+ *   o strValue (I) - Channel translation value.
  * Returns:
  *   The integer option value.
  *----------------------------------------------------------------------------- */
@@ -682,7 +847,7 @@ ParseTranslationOption (strValue)
  * direction.
  *
  * Parameters:
- *   o value - Integer channel translation value.
+ *   o value (I) - Integer channel translation value.
  * Returns:
  *   The string option value.
  *----------------------------------------------------------------------------
@@ -715,25 +880,20 @@ FormatTranslationOption (value)
  *   C-friendly front end to Tcl_GetChannelOption.
  *
  * Parameters:
- *   o interp - Error message are returned in result
- *   o channel - Channel to get the option for.
- *   o optionName - One of the TCLX_COPT_* defines.
- *   o valuePtr - Value is returned here.
+ *   o channel (I) - Channel to get the option for.
+ *   o optionName (I) - One of the TCLX_COPT_* defines.
  * Returns:
- *   TCL_OK or TCL_ERROR.
- * FIX: Maybe drop these.
+ *   The integer option value define by TclX.
  *-----------------------------------------------------------------------------
  */
 int
-TclX_GetChannelOption (interp, channel, option, valuePtr)
-    Tcl_Interp *interp;
+TclX_GetChannelOption (channel, option)
     Tcl_Channel channel;
     int         option;
-    int        *valuePtr;
 {
-    char          *strOption;
-    Tcl_DString    strValue;
-    int            value = 0;
+    char *strOption;
+    Tcl_DString strValue;
+    int value;
 
     Tcl_DStringInit (&strValue);
 
@@ -754,11 +914,8 @@ TclX_GetChannelOption (interp, channel, option, valuePtr)
         goto fatalError;
     }
 
-    if (Tcl_GetChannelOption (interp, channel, strOption,
-                              &strValue) != TCL_OK) {
-        Tcl_DStringFree (&strValue);
-        return TCL_ERROR;
-    }
+    if (Tcl_GetChannelOption (channel, strOption, &strValue) != TCL_OK)
+        goto fatalError;
 
     switch (option) {
       case TCLX_COPT_BLOCKING:
@@ -813,11 +970,10 @@ TclX_GetChannelOption (interp, channel, option, valuePtr)
       }
     }
     Tcl_DStringFree (&strValue);
-    *valuePtr = value;
-    return TCL_OK;
+    return value;
 
   fatalError:
-    panic ("TclX_GetChannelOption bug");  /* FIX: return error. */
+    panic ("TclX_GetChannelOption bug");
     return 0;  /* Not reached */
 }
 
@@ -827,10 +983,10 @@ TclX_GetChannelOption (interp, channel, option, valuePtr)
  *   C-friendly front end to Tcl_SetChannelOption.
  *
  * Parameters:
- *   o interp - Errors returned in result.
- *   o channel - Channel to set the option for.
- *   o option - One of the TCLX_COPT_* defines.
- *   o value - Value to set the option to (integer define).  Note, if
+ *   o interp (I) - Errors returned in result.
+ *   o channel (I) - Channel to set the option for.
+ *   o option (I) - One of the TCLX_COPT_* defines.
+ *   o value (I) - Value to set the option to (integer define).  Note, if
  *     this is translation, it can either be the read and write directions
  *     masked together or a single value.
  * Result:
@@ -923,8 +1079,8 @@ TclX_SetChannelOption (interp, channel, option, value)
  *   Interface to Tcl_Join path to join only two files.
  *
  * Parameters:
- *   o path1, path2 - File paths to join.
- *   o joinedPath - DString buffere that joined path is returned in.
+ *   o path1, path2 (I) - File paths to join.
+ *   o joinedPath (O) - DString buffere that joined path is returned in.
  *     must be initialized.
  * Returns:
  *   A pointer to joinedPath->string.
@@ -944,263 +1100,3 @@ TclX_JoinPath (path1, path2, joinedPath)
 
     return joinedPath->string;
 }
-
-
-/*-----------------------------------------------------------------------------
- * TclX_WrongArgs --
- *
- *   Easily create "wrong # args" error messages.
- *
- * Parameters:
- *   o commandNameObj - Object containing name of command (objv[0])
- *   o string - Text message to append.
- * Returns:
- *   TCL_ERROR
- *-----------------------------------------------------------------------------
- */
-int
-TclX_WrongArgs (interp, commandNameObj, string)
-    Tcl_Interp  *interp;
-    Tcl_Obj     *commandNameObj;
-    char        *string;
-{
-    char    *commandName;
-    Tcl_Obj *resultPtr = Tcl_GetObjResult (interp);
-    int      commandLength;
-
-    commandName = Tcl_GetStringFromObj (commandNameObj, &commandLength);
-
-    Tcl_AppendStringsToObj (resultPtr,
-			    tclXWrongArgs,
-			    commandName,
-			    (char *)NULL);
-
-    if (*string != '\0') {
-	Tcl_AppendStringsToObj (resultPtr, " ", string, (char *)NULL);
-    }
-    return TCL_ERROR;
-}
-
-
-/*-----------------------------------------------------------------------------
- * TclX_AppendObjResult --
- *
- *   Append a variable number of strings onto the object result already
- * present for an interpreter.  If the object is shared, the current contents
- * are discarded.
- *
- * Parameters:
- *   o interp - Interpreter to set the result in.
- *   o args - Strings to append, terminated by a NULL.
- *-----------------------------------------------------------------------------
- */
-void
-TclX_AppendObjResult TCL_VARARGS_DEF (Tcl_Interp *, arg1)
-{
-    Tcl_Interp *interp;
-    Tcl_Obj *resultPtr;
-    va_list argList;
-    char *string;
-
-    interp = TCL_VARARGS_START (Tcl_Interp *, arg1, argList);
-    resultPtr = Tcl_GetObjResult (interp);
-
-    if (Tcl_IsShared(resultPtr)) {
-        resultPtr = Tcl_NewStringObj((char *)NULL, 0);
-        Tcl_SetObjResult(interp, resultPtr);
-    }
-
-    TCL_VARARGS_START(Tcl_Interp *,arg1,argList);
-    while (1) {
-        string = va_arg(argList, char *);
-        if (string == NULL) {
-            break;
-        }
-        Tcl_AppendToObj (resultPtr, string, -1);
-    }
-    va_end(argList);
-}
-
-
-/*-----------------------------------------------------------------------------
- * TclX_IsNullObj --
- *
- *   Check if an object is {}, either in list or zero-lemngth string form, with
- * out forcing a conversion.
- *
- * Parameters:
- *   o objPtr - Object to check.
- * Returns:
- *   True if NULL, FALSE if not.
- *-----------------------------------------------------------------------------
- */
-int
-TclX_IsNullObj (objPtr)
-    Tcl_Obj *objPtr;
-{
-    static Tcl_ObjType *listType = NULL, *stringType = NULL;
-    int length;
-    
-    /*
-     * Only get types once, as they must be static.
-     */
-    if (listType == NULL) {
-        listType = Tcl_GetObjType ("list");
-        stringType = Tcl_GetObjType ("string");
-    }
-
-    if (objPtr->typePtr == NULL) {
-        return (objPtr->length == 0);
-    } else {
-        if (objPtr->typePtr == listType) {
-            Tcl_ListObjLength (NULL, objPtr, &length);
-            return (length == 0);
-        } else if (objPtr->typePtr == stringType) {
-            Tcl_GetStringFromObj (objPtr, &length);
-            return (length == 0);
-        }
-    }
-    Tcl_GetStringFromObj (objPtr, &length);
-    return (length == 0);
-}
-
-
-/*-----------------------------------------------------------------------------
- * TclX_SaveResultErrorInfo --
- *
- *   Saves the Tcl interp result plus errorInfo and errorCode in a structure.
- *
- * Parameters:
- *   o interp - Interpreter to save state for.
- * Returns:
- *   A list object containing the state.
- *-----------------------------------------------------------------------------
- */
-Tcl_Obj *
-TclX_SaveResultErrorInfo (interp)
-    Tcl_Interp  *interp;
-{
-    Tcl_Obj *saveObjv [4];
-    Tcl_Obj *listObj;
-
-    long flags = ((Interp *)interp)->flags &
-	(ERR_ALREADY_LOGGED | ERR_IN_PROGRESS | ERROR_CODE_SET);
-
-    saveObjv [0] = Tcl_DuplicateObj (Tcl_GetObjResult (interp));
-    
-    saveObjv [1] = Tcl_GetVar2Ex(interp, ERRORINFO, NULL, TCL_GLOBAL_ONLY);
-    if (saveObjv [1] == NULL) {
-        saveObjv [1] = Tcl_NewObj ();
-    }
-
-    saveObjv [2] = Tcl_GetVar2Ex(interp, ERRORCODE, NULL, TCL_GLOBAL_ONLY);
-    if (saveObjv [2] == NULL) {
-        saveObjv [2] = Tcl_NewObj ();
-    }
-
-    saveObjv [3] = Tcl_NewLongObj(flags);
-
-    Tcl_IncrRefCount(listObj = Tcl_NewListObj (4, saveObjv));
-
-    return listObj;
-}
-
-
-/*-----------------------------------------------------------------------------
- * TclX_RestoreResultErrorInfo --
- *
- *   Restores the Tcl interp state from TclX_SaveResultErrorInfo.
- *
- * Parameters:
- *   o interp - Interpreter to save state for.
- *   o saveObjPtr - Object returned from TclX_SaveResultErrorInfo.  Ref count
- *     will be decremented.
- *-----------------------------------------------------------------------------
- */
-void
-TclX_RestoreResultErrorInfo (interp, saveObjPtr)
-    Tcl_Interp *interp;
-    Tcl_Obj    *saveObjPtr;
-{
-    Tcl_Obj **saveObjv;
-    int saveObjc;
-    long flags;
-
-    if ((Tcl_ListObjGetElements (NULL, saveObjPtr, &saveObjc,
-                                 &saveObjv) != TCL_OK) ||
-        (saveObjc != 4) ||
-        (Tcl_GetLongFromObj (NULL, saveObjv[3], &flags) != TCL_OK)) {
-	/*
-	 * This should never happen
-	 */
-        panic ("invalid TclX result save object");
-    }
-
-    Tcl_SetVar2Ex(interp, ERRORCODE, NULL, saveObjv[2], TCL_GLOBAL_ONLY);
-    Tcl_SetVar2Ex(interp, ERRORINFO, NULL, saveObjv[1], TCL_GLOBAL_ONLY);
-
-    Tcl_SetObjResult (interp, saveObjv[0]);
-
-    ((Interp *)interp)->flags |= flags;
-
-    Tcl_DecrRefCount (saveObjPtr);
-}
-
-
-/*-----------------------------------------------------------------------------
- * TclX_ShellExit --
- *
- *   Handles exiting a shell.  Normally just does an exit, but deletes the
- * interp if compiled with TCL_MEM_DEBUG or the Tcl global
- * TCLXENV(deleteInterpAtShellExit) is a true boolean value.  Deleting the
- * interp before exiting is useful for tracking down memory leaks.  If 
- * TCL_MEM_DEBUG is set, a list of allocated memory is written to 
- * "tclmem.$pid.lst" on Unix systems.
- *
- * Parameters:
- *   o interp - Interpreter.
- *   o exitCode - Code to pass to exit call.
- *-----------------------------------------------------------------------------
- */
-void
-TclX_ShellExit (interp, exitCode)
-    Tcl_Interp *interp;
-    int         exitCode;
-{
-#if defined(TCL_MEM_DEBUG)
-    /*
-     * On Unix, Tcl_Exit will dump a list of leaked ckalloc's if this
-     * variable is set.  On Win32, we can't set it, since its in another
-     * DLL.
-     */
-#ifndef __WIN32__
-    extern char *tclMemDumpFileName;
-    static char dumpFileName [128];
-    sprintf (dumpFileName, "tclmem.%d.lst", getpid ());
-    tclMemDumpFileName = dumpFileName;
-#endif
-    Tcl_DeleteInterp (interp);
-    Tcl_Exit (0);
-#else
-    Tcl_Obj *varValue;
-    int deleteInterp;
-
-    /*
-     * If TCLXENV(deleteInterpAtShellExit) is a true boolean, delete
-     * interpreter.
-     */
-    deleteInterp = FALSE;
-    varValue = Tcl_GetVar2Ex(interp, "TCLXENV", "deleteInterpAtShellExit",
-                             TCL_GLOBAL_ONLY);
-    if (varValue != NULL) {
-        Tcl_GetBooleanFromObj (NULL, varValue, &deleteInterp);
-    }
-    
-    if (deleteInterp) {
-        Tcl_DeleteInterp (interp);
-        Tcl_Exit (0);
-    } else {
-        Tcl_Exit (0);
-    }
-#endif    
-}
