@@ -16,36 +16,64 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: testlib.tcl,v 7.0 1996/06/16 05:33:03 markd Exp $
+# $Id: testlib.tcl,v 8.0 1996/11/21 00:25:07 markd Exp $
 #------------------------------------------------------------------------------
 #
 
 # Save the unknown command in a variable SAVED_UNKNOWN.  To get it back, eval
 # that variable.  Don't do this more than once.
 
-global SAVED_UNKNOWN TCL_PROGRAM env TEST_ERROR_INFO
+global SAVED_UNKNOWN TCL_PROGRAM env TEST_ERROR_INFO tcl_platform testXConfig
 
 if [info exists env(TEST_ERROR_INFO)] {
     set TEST_ERROR_INFO 1
 }
+# Check configuration information that will determine which tests
+# to run.  To do this, create an array testXConfig.  Each element
+# has a 0 or 1 value, and the following elements are defined:
+#	unixOnly -	1 means this is a UNIX platform, so it's OK
+#			to run tests that only work under UNIX.
+#	pcOnly -	1 means this is a PC platform, so it's OK to
+#			run tests that only work on PCs.
+#	unixOrPc -	1 means this is a UNIX or PC platform.
+#	tempNotPc -	The inverse of pcOnly.  This flag is used to
+#			temporarily disable a test.
+
+catch {unset testXConfig}
+if {$tcl_platform(platform) == "unix"} {
+    set testXConfig(unixOnly) 1
+    set testXConfig(tempNotPc) 1
+} else {
+    set testXConfig(unixOnly) 0
+} 
+if {$tcl_platform(platform) == "windows"} {
+    set testXConfig(pcOnly) 1
+} else {
+    set testXConfig(pcOnly) 0
+}
+set testXConfig(unixOrPc) [expr $testXConfig(unixOnly) || $testXConfig(pcOnly)]
 
 #
 # Save path to Tcl program to exec, use it when running children in the
-# tests.
+# tests.  Order of checking:
+#   o Environment variable TCL_PROGRAM
+#   o If $argv0 can be found, use it.  Generate an absolute path.
+#   o Use a name "tcl", hopefully on the path.
+# Then normallize path for the platform.
 #
 if ![info exists TCL_PROGRAM] {
     if [info exists env(TCL_PROGRAM)] {
        set TCL_PROGRAM $env(TCL_PROGRAM)
     } else {
-       puts stderr ""
-       puts stderr {WARNING: No environment variable TCL_PROGRAM,}
-       puts stderr {         assuming "tcl" as program to use for}
-       puts stderr {         running subprocesses in the tests.}
-       puts stderr ""
-       set TCL_PROGRAM tcl
+        set TCL_PROGRAM [info nameofexecutable]
+        puts "    * WARNING: No environment variable TCL_PROGRAM, Using the"
+        puts "    * command \"$TCL_PROGRAM\""
+        puts "    * as the program to use for running subprocesses in the tests."
     }
 }
 
+set TCL_PROGRAM [eval file join [file split $TCL_PROGRAM]]
+
 if {([info command unknown] == "") && ![info exists SAVED_UNKNOWN]} {
     error "can't find either unknown or SAVED_UNKNOWN"
 }
@@ -96,7 +124,17 @@ proc OutTestError {test_name test_description contents_of_test
 # Routine to execute tests and compare to expected results.
 #
 proc Test {test_name test_description contents_of_test passing_int_result
-           passing_result} {
+           passing_result {constraints {}}} {
+    global testXConfig
+
+    # Check constraints to see if we should run this test.
+    foreach constraint $constraints {
+        if {![info exists testXConfig($constraint)] ||
+            !$testXConfig($constraint)} {
+                return
+        }
+    }
+
     set int_result [catch {uplevel $contents_of_test} result]
 
     if {($int_result != $passing_int_result) ||
@@ -139,39 +177,62 @@ proc GenRec {id} {
 
 # Proc to fork and exec child that loops until it gets a signal.
 # Can optionally set its pgroup.  Wait till child has actually execed or
-# kill breaks on some systems (i.e. AIX).
+# kill breaks on some systems (i.e. AIX).  Windows is a drag, since the
+# command line parsing is really dumb.  Pass it in a file instead.
 
 proc ForkLoopingChild {{setPGroup 0}} {
-    global TCL_PROGRAM
-    close [open CHILD.RUN w]
+    global TCL_PROGRAM tcl_platform
+
+    set childProg {file delete CHILD.RUN; catch {while {1} {sleep 1}}; exit 10}
+
+    # Create semaphore (it also contains the program to run for windows).
+    set fh [open CHILD.RUN w]
+    puts $fh $childProg
+    close $fh
     flush stdout
     flush stderr
-    set newPid [fork]
-    if {$newPid != 0} {
-        # Wait till the child is actually running.
-        while {[file exists CHILD.RUN]} {
-            sleep 1
+
+    if [cequal $tcl_platform(platform) unix] {
+        set newPid [fork]
+        if {$newPid == 0} {
+            if $setPGroup {
+                id process group set
+            }
+            catch {
+                execl $TCL_PROGRAM [list -qc $childProg]
+            } msg
+            puts stderr "execl failed (ForkLoopingChild): $msg"
+            exit 1
         }
-        return $newPid
     }
-    if $setPGroup {
-        id process group set
+    if [cequal $tcl_platform(platform) windows] {
+        if $setPGroup {
+            error "setpgroup not supported on windows"
+        }
+        set newPid [execl $TCL_PROGRAM [list -q CHILD.RUN]]
     }
-    catch {
-        execl $TCL_PROGRAM \
-            {-qc {unlink CHILD.RUN; catch {while {1} {sleep 1}}; exit 10}}
-    } msg
-    puts stderr "execl failed (ForkLoopingChild): $msg"
-    exit 1
+        
+        
+    # Wait till the child is actually running.
+    while {[file exists CHILD.RUN]} {
+        sleep 1
+    }
+    return $newPid
 }
 
 #
 # Create a file.  If the directory doesn't exist, create it.
 #
-proc tcltouch file {
-    if ![file exists [file dirname $file]] {
-        mkdir -path [file dirname $file]
-    }
+proc TestTouch file {
+    file mkdir [file dirname $file]
     close [open $file w]
 }
 
+#
+# Remove files and directories with out errors.
+#
+proc TestRemove args {
+    foreach f $args {
+        catch {file delete -force $f}
+    }
+}
