@@ -12,7 +12,7 @@
  * software for any purpose.  It is provided "as is" without express or
  * implied warranty.
  *-----------------------------------------------------------------------------
- * $Id: tclXwinPort.h,v 7.0 1996/06/16 05:33:39 markd Exp $
+ * $Id: tclXwinPort.h,v 8.0 1996/11/21 00:25:26 markd Exp $
  *-----------------------------------------------------------------------------
  */
 
@@ -21,6 +21,15 @@
 
 #include "tclWinPort.h"
 
+#include <direct.h>
+
+/*
+ * Types needed for fstat, but are not directly supported (we emulate).  If
+ * defined before tclWinPort.h is include, it will define the access macros.
+ */
+#define S_IFIFO  _S_IFIFO               /* pipe */
+#define S_IFSOCK 0140000                /* socket */
+
 /*
  * OS feature definitons.
  */
@@ -31,7 +40,8 @@
 #define NO_RANDOM  /* uses compat */
 #define NO_SIGACTION
 #define NO_SYS_SELECT_H
-#define NO_TRUNCATE
+#define NO_TRUNCATE    /* FIX: Are we sure there is no way to truncate???*/
+#define NO_FTRUNCATE
 #define RETSIGTYPE void
 
 #include <math.h>
@@ -45,9 +55,21 @@
  * it delays signals.
  */
 #define TCLX_WAITPID(pid, status, options) Tcl_WaitPid (pid, status, options)
+
+#define bcopy(from, to, length)    memmove((to), (from), (length))
+
 /*
- * Handle used to access directories.
+ * Compaibility functions.
  */
-#define TCLX_DIRHANDLE HANDLE
+extern long
+random (void);
+
+extern void
+srandom (unsigned int x);
+
+extern int
+getopt (int           nargc,
+        char * const *nargv,
+        const char   *ostr);
 
 #endif
