@@ -30,7 +30,7 @@
 # software for any purpose.  It is provided "as is" without express or
 # implied warranty.
 #------------------------------------------------------------------------------
-# $Id: bldmanhelp.tcl,v 7.0 1996/06/16 05:34:29 markd Exp $
+# $Id: bldmanhelp.tcl,v 8.0 1996/11/21 00:25:50 markd Exp $
 #------------------------------------------------------------------------------
 #
 
@@ -72,18 +72,18 @@ proc CopyManPage {manPage outFH} {
         return
     }
     while {[gets $fh line] >= 0} {
-        case [lindex $line 0] {
-            {.so} {
+        switch -glob -- $line {
+            .so* {
                 CopyManPage [lindex $line 1] $outFH
             }
-            {.SH} {
+            .SH* {
                 puts $outFH $line
                 if {[lindex $line 1] == "NAME"} {
                     set line [ProcessNameSection $fh $outFH]
                     puts $outFH $line
                 }
             }
-            {.HS .BS .BE .VS .VE .TH} {
+            .HS* - .BS* - .BE* - .VS* - .VE* - .TH* {
             }
             default {
                 if !$skipSection {
@@ -147,7 +147,7 @@ GenInputFile $docDir $manInfoTbl $tmpFile
 
 buildhelp $helpDir $brief [list $tmpFile]
 
-unlink $tmpFile
+file delete $tmpFile
 
 if $gotErrors {
     puts stderr "Errors occured processing manual files"
